/********************************************************************************************************
 * @file     llms.h
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     May. 22, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#ifndef LLMS_H_
#define LLMS_H_

#include "stack/ble/llms/llms_config.h"
#include "stack/ble/llms/llms_conn.h"
#include "stack/ble/llms/llms_slot.h"






/******************************* Macro & Enumeration & Structure Definition for Stack Begin, user can not use!!!!  *****/
//TODO:  all debug MACRO should remove at last
#define 		DBG_SLOT_CALCULTE_TIMING_EN						0
#define			DBG_DECRYPTION_ERR_EN							0


#define 	    BLMS_CONN_TIMING_EXTEND							1

#define			CONNECT_COMPLETE_CALLBACK_IN_MAINLOOP			1




#define			BLMS_STATE_NONE									0
#define			BLMS_STATE_ADV									BIT(1)
#define			BLMS_STATE_SCAN_START							BIT(2)
#define			BLMS_STATE_SCAN_POST							BIT(3)
#define			BLMS_STATE_BTX_S								BIT(4)
#define			BLMS_STATE_BTX_E								BIT(5)
#define			BLMS_STATE_BRX_S								BIT(6)
#define			BLMS_STATE_BRX_E								BIT(7)

#define			BLMS_STATE_SCAN_INSERT							BIT(8)


#define 		BLMS_FLG_RF_CONN_DONE 		 					(FLD_RF_IRQ_CMD_DONE  | FLD_RF_IRQ_FIRST_TIMEOUT | FLD_RF_IRQ_RX_TIMEOUT | FLD_RF_IRQ_RX_CRC_2)

//TODO: delete all debug codes after functions are tested OK
//#define 		BLMS_ERR_DEBUG(x)								do{irq_disable();write_reg32(0x40000, (x));while(1);}while(0)
#define 		BLMS_ERR_DEBUG(x)





#define			SYS_IRQ_TRIG_SLOT_TASK							0         //Scan/ ADV /BRX start /BTX start
#define			SYS_IRQ_TRIG_BTX_POST							BIT(0)
#define			SYS_IRQ_TRIG_BRX_POST							BIT(1)
#define			SYS_IRQ_TRIG_SCAN_POST							BIT(2)

#define			SYS_IRQ_TRIG_BRX_START							BIT(3)   //only slave high duty mode use



/////////////////////////////////////////////////////////////////////////////
//TX settle time
#define			LL_ADV_TX_SETTLE				74
#define 		LL_SCAN_TX_SETTLE				56
#define 		LL_SLAVE_TX_SETTLE				75
#define 		LL_MASTER_TX_SETTLE				76

#define			LL_ADV_TX_STL_2M				84
#define			LL_ADV_TX_STL_CODED				116

#define 		LL_SLAVE_TX_STL_2M				86
#define 		LL_SLAVE_TX_STL_CODED			118

#define			LL_MASTER_TX_STL_2M				86
#define			LL_MASTER_TX_STL_CODED			119

#if (NEW_CODE_STRUCT)
typedef struct{
	u8  durationSlotNum;
	u8  rsvd[3];

	u16 positionOffset[24]; ///position offset //now support max 24 master
}master_posOffset_t;

extern _attribute_aligned_(4) master_posOffset_t   master_posOffset;
#endif


extern u32 				Crc24Lookup[16];

#if (FIX_HW_CRC24_EN)
	#define		CRC_MATCH8(md,ms)	(md[0]==ms[0] && md[1]==ms[1] && md[2]==ms[2])
#endif


my_fifo_t			blt_rxfifo;
u8					blt_rxfifo_b[];

my_fifo_t			blt_txfifo;
u8					blt_txfifo_b[];

extern my_fifo_t	hci_tx_fifo;

_attribute_aligned_(4)
typedef struct {
	u8		macAddress_public[6];
	u8		macAddress_random[6];   //host may set this
}ll_mac_t;
extern ll_mac_t  bltMac;

#if (NEW_CODE_STRUCT)
_attribute_aligned_(4)
typedef struct {
	u8		adv_en;
	u8		scan_en;
	u8		initiate_en;
	u8		state_chng;  // ADV or Scan state change


	u8		max_master_num;
	u8		max_slave_num;
	u8		cur_master_num;
	u8		cur_slave_num;

	u32     connSync;                //if more than 8 connections, u8 -> u16

	u32		newConn_forbidden_slave; //u8 -> u16

	u8      rsvd2;
	u8      initiating_flag;
	u8		adv_scanReq_connReq;
	u8		llblock;

	u32		connectEvt_mask;  ///u8 -> u32
	u32		disconnEvt_mask;  /// u8 -> u32
	u32		conupdtEvt_mask;  //conn_update  u8 -> u32
	u32		phyupdtEvt_mask;  /// u8 ->u32

	u8		master_connInter;   	// "u8" is enough, 320 mS most
	u8		half_connInter;
	u8		quarter_connInter;
	u8      rsvd0;
//	u8		slotNum_whole_inter;

	u32     slotNum_whole_inter;


	u8		slotNum_half_inter;
	u8		slotNum_quarter_inter;
	u8		drop_rx_data;
	u8		scanpost_clear_rx_status;

	u8  	state_flag;  // adv and scan are both close, open need judge
	u8		connSync_master;
	u8		connSync_slave;
	u8      rsvd1;

	u32		newConn_forbidden_master; //u8->u16

	u32		slot_idx_1st_btx_mark;   // slot_type
	u8		slot_offset_mark[BLMS_MAX_CONN_MASTER_NUM]; //maybe 1 or 2, so need lay it at last


} st_llms_para_t;
#else
_attribute_aligned_(4)
typedef struct {
	u8		adv_en;
	u8		scan_en;
	u8		initiate_en;
	u8		state_chng;  // ADV or Scan state change


	u8		max_master_num;
	u8		max_slave_num;
	u8		cur_master_num;
	u8		cur_slave_num;


	u8		connSync;				//if more than 8 connections, u8 -> u16
	u8		newConn_forbidden_slave;
	u8		adv_scanReq_connReq;
	u8		llblock;

	u8		connectEvt_mask;
	u8		disconnEvt_mask;
	u8		conupdtEvt_mask;  //conn_update
	u8 		phyupdtEvt_mask;

	u8		master_connInter;   	// "u8" is enough, 320 mS most
	u8		half_connInter;
	u8		quarter_connInter;
	u8		slotNum_whole_inter;

	u8		slotNum_half_inter;
	u8		slotNum_quarter_inter;
	u8		drop_rx_data;
	u8		scanpost_clear_rx_status;

	u8  	state_flag;  // adv and scan are both close, open need judge
	u8		connSync_master;
	u8		connSync_slave;
	u8		newConn_forbidden_master;

	u32		slot_idx_1st_btx_mark;   // slot_type
	u8		slot_offset_mark[BLMS_MAX_CONN_MASTER_NUM]; //maybe 1 or 2, so need lay it at last


} st_llms_para_t;
#endif

extern _attribute_aligned_(4)	st_llms_para_t  blmsParam;



extern	volatile	u32	blms_state;
extern	volatile    int blm_btxbrx_state;
extern				u8	blms_tx_empty_packet[];
extern	volatile	int		blm_btxbrx_busy;
extern	volatile	u32	systick_irq_trigger;
extern	volatile	u32 rx_tick_now;
extern	volatile	u8	btxbrx_post_dataBusy_flag;
extern	u32 			btxbrx_post_dataBusy_clock_tick;

typedef     int (*llms_blt_main_loop_post_callback_t)(void);
typedef 	int (*llms_host_mainloop_callback_t)(void);
typedef 	int (*llms_enc_done_callback_t)(u16 connHandle, u8 status, u8 enc_enable);
typedef 	int (*llms_conn_complete_handler_t)(u16 conn, u8 *p);
typedef 	int (*llms_conn_terminate_handler_t)(u16 conn, u8 *p);

extern llms_host_mainloop_callback_t 	llms_host_main_loop_cb;
extern llms_enc_done_callback_t 	 	llms_encryption_done_cb;
extern llms_conn_complete_handler_t 	llms_connComplete_handler;
extern llms_conn_terminate_handler_t 	llms_connTerminate_handler;
/******************************* Macro & Enumeration & Structure Definition for Stack End ******************************/










/******************************* Macro & Enumeration variables for User Begin ******************************************/

////////////////// Telink defined LinkLayer Event Callback  ////////////////////////
typedef void (*blt_event_callback_t)(u8 e, u8 *p, int n);

#define 		BLT_EV_MAX_NUM						10

#define			BLT_EV_FLAG_ADV						0
#define			BLT_EV_FLAG_ADV_DURATION_TIMEOUT	1
#define			BLT_EV_FLAG_SCAN_RSP				2
#define			BLT_EV_FLAG_LL_REJECT_IND		    3
#define			BLT_EV_FLAG_RX_DATA_ABANDOM			4
#define			BLT_EV_FLAG_GPIO_EARLY_WAKEUP		5
#define			BLT_EV_FLAG_SUSPEND_ENTER			6
#define			BLT_EV_FLAG_SUSPEND_EXIT			7

/******************************* Macro & Enumeration variables for User End ********************************************/








/******************************* Stack Interface Begin, user can not use!!! ********************************************/
void 		irq_ble_master_slave_handler(void);
int 		blt_llms_procPendingEvent(void);
//int 		blt_llms_procConnectionEvent(u16 connHandle,  st_llms_conn_t* pc); //


void		smemset4(int * dest, int val, int len);



void 		blt_llms_registerHostMainloopCallback (llms_host_mainloop_callback_t cb);
void 		blt_llms_registerConnectionEncryptionDoneCallback(llms_enc_done_callback_t  cb);
void 		blt_llms_registerConnectionCompleteHandler(llms_conn_complete_handler_t  handler);
void 		blt_llms_registerConnectionTerminateHandler(llms_conn_terminate_handler_t  handler);

static inline void blt_llms_setRxBlockSts(bool block)
{
	blmsParam.llblock = block;
}

static inline u8 blt_llms_getRxBlockSts(void)
{
	return blmsParam.llblock;
}

u8 			blt_llms_push_hold_data(u16 connHandle);
void  		blt_llms_setEncryptionBusy(u16 connHandle, u8 enc_busy);
int  		blt_llms_isEncryptionBusy(u16 connHandle);
u8 			blt_llms_pushTxfifo_hold (u16 connHandle, u8 *p);
u8  		blt_llms_smpPushEncPkt (u16 connHandle, u8 type);
void 		blt_llms_smpSecurityProc(u16 connHandle);
void 		blt_llms_sdk_main_loop (void);
void 		blt_csa1_calculateChannelTable(u8* chm, u8 hop, u8 *ptbl);
u32 		blt_llms_conn_calc_access_addr(void);
u32 		blt_llms_conn_calc_access_addr_v2(void);
void 		blt_llms_initDataLengthExtension (void);

u8			blt_llms_getCurrentState(void);
u8 			blt_llms_getOwnAddrType(u16 connHandle);
u8*			blt_llms_get_OwnMacAddr(u16 connHandle, u8 addr_type);
/******************************* Stack Interface End *******************************************************************/










/******************************* User Interface  Begin *****************************************************************/
void 		blc_llms_initBasicMCU (void);
void 		blc_llms_initStandby_module (u8 *public_adr);
void 		blc_llms_initMultiMasterMultiSlave_module(void);
int 		blc_llms_is_handle_valid (u16 handle);
u8 			blc_llms_isAllRolesNotConnected(void);
ble_sts_t 	blc_llms_setRandomAddr(u8 *randomAddr);

#if (NEW_CODE_STRUCT)
 void        blc_llms_setMasterSlotPosForM16S3(void);  ///set master slot position
 void        blc_llms_setMasterSlotPosForM24S3(void);  ///set master slot position
#endif


void 		blc_llms_registerTelinkControllerEventCallback (u8 e, blt_event_callback_t p);

void		blc_llms_resetSlotTiming(void);	// use in connectionless state

/******************************* User Interface  End  ******************************************************************/





#endif /* LLMS_H_ */
