/********************************************************************************************************
 * @file     llms_init.h
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     May. 22, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#ifndef LLMS_INIT_H_
#define LLMS_INIT_H_


#include "stack/ble/llms/llms_config.h"


/******************************* Macro & Enumeration & Structure Definition for Stack Begin, user can not use!!!!  *****/


//can not be "1", will conflict with " blms_create_connection & CONN_IDX_MASK"
#define			BLMS_CONNECTION_REQ						BIT(15)
#define			BLMS_CONNECTION_REQ_PENDING				BIT(14)

#define  		BLMS_WINSIZE							7


#define			BLMS_DIRECT_INIT_EN						1


_attribute_aligned_(4)
typedef struct {
	u8	conn_policy;
	u8	conn_advType;

	u8	conn_mac[6];

	u8  own_addr_type;
	u8  rsvd[3];
}st_llms_init_t;

extern	_attribute_aligned_(4) st_llms_init_t  blmsInit;

extern	volatile	int blms_create_connection;
extern	u32 blms_timeout_connectDevice;
extern	u32	blms_tick_connectDevice;


extern	rf_packet_ll_init_t	pkt_llms_init;


/******************************* Macro & Enumeration & Structure Definition for Stack End ******************************/







/******************************* Macro & Enumeration variables for User Begin ******************************************/



/******************************* Macro & Enumeration variables for User End ********************************************/





/******************************* Stack Interface Begin, user can not use!!! ********************************************/
int  		blt_llms_procInitPkt(u8 *raw_pkt);

void 		blt_llms_procInitiateConnectionTimeout(void);

static inline u32 rf_get_ble_access_code_16to32 (u16 code)
{
	u32 r = 0;
	int i;
	for (i=0; i<4; i++) {
		r = r << 2;
		r |= code & BIT(i) ? 1 : 2;
	}

	for (i=4; i<8; i++) {  //no more 24 transitions
		r = r << 4;
		r |= code & BIT(i) ? 0x0c : 0x03;
	}

	for (i=12; i<16; i++) {
		r = r << 2;
		r |= code & BIT(i) ? 1 : 2;
	}

	return r;
}

/******************************* Stack Interface End *******************************************************************/







/******************************* User Interface  Begin *****************************************************************/
ble_sts_t 	blc_llms_createConnection( u16 scan_interval, u16 scan_window, init_fp_type_t initiator_filter_policy,
										u8 adr_type, 	  u8  *mac, 	   u8 			  own_adr_type,
									   u16 conn_min, 	  u16 conn_max,    u16 			  conn_latency, 				u16 timeout,
									   u16 ce_min, 		  u16 ce_max );

ble_sts_t 	blc_llms_createConnectionCancel (void);

ble_sts_t 	blc_llms_setCreateConnectionTimeout (u32 timeout_ms);


/******************************* User Interface  End  ******************************************************************/



#endif /* LLMS_INIT_H_ */
