/********************************************************************************************************
 * @file     llms_slot.h
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     May. 22, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#ifndef LLMS_SLOT_H_
#define LLMS_SLOT_H_

#include "llms_config.h"

/******************************* Macro & Enumeration & Structure Definition for Stack Begin, user can not use!!!!  *****/


#define			SLOT_MAX_NUM									128


/*******************************************************************************
 *
 * BIT(7)	BIT(6)	BIT(5)	BIT(4)	   (BIT(3) BIT(2)  BIT(1) BIT(0) )
 *   M		  S		 SCAN	 ADV		  		conn_handle_idx
 ******************************************************************************/
#define			SLOT_TASK_ADV									BIT(4)
#define			SLOT_TASK_SCAN									BIT(5)
#define			SLOT_TASK_SLAVE									BIT(6)
#define			SLOT_TASK_MASTER								BIT(7)

#define 		SLOT_TASK_CONN									(SLOT_TASK_SLAVE | SLOT_TASK_MASTER)



#if(NEW_CODE_STRUCT)
	#define			CONN_IDX_MASK									31
#else
	#define			CONN_IDX_MASK									15    //15 is very safe. cause total connection number will not exceed 15
#endif






	#define			SLOT_TASK_MSK_CONN_0							BIT(0)

#if (CONN_MAX_NUM_CONFIG == CONN_MAX_NUM_M1_S1)

	#define			SLOT_TASK_MSK_MASTER0							BIT(0)
	#define			SLOT_TASK_MSK_SLAVE0							BIT(1)

	#define 		SLOT_TASK_MSK_MASTER							SLOT_TASK_MSK_MASTER0
	#define 		SLOT_TASK_MSK_SLAVE								SLOT_TASK_MSK_SLAVE0

	#define 		SLOT_TASK_MSK_CONN								( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_SLAVE0 )

#elif (CONN_MAX_NUM_CONFIG == CONN_MAX_NUM_M2_S2)

	#define			SLOT_TASK_MSK_MASTER0							BIT(0)
	#define			SLOT_TASK_MSK_MASTER1							BIT(1)
	#define			SLOT_TASK_MSK_SLAVE0							BIT(2)
	#define			SLOT_TASK_MSK_SLAVE1							BIT(3)


	#define 		SLOT_TASK_MSK_MASTER							( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 )
	#define 		SLOT_TASK_MSK_SLAVE								( SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1 )


	#define 		SLOT_TASK_MSK_CONN								( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 | \
																	  SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1 )
#elif (CONN_MAX_NUM_CONFIG == CONN_MAX_NUM_M4_S3)

	#define			SLOT_TASK_MSK_MASTER0							BIT(0)
	#define			SLOT_TASK_MSK_MASTER1							BIT(1)
	#define			SLOT_TASK_MSK_MASTER2							BIT(2)
	#define			SLOT_TASK_MSK_MASTER3							BIT(3)
	#define			SLOT_TASK_MSK_SLAVE0							BIT(4)
	#define			SLOT_TASK_MSK_SLAVE1							BIT(5)
	#define			SLOT_TASK_MSK_SLAVE2							BIT(6)


	#define 		SLOT_TASK_MSK_MASTER							( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 |  \
																	  SLOT_TASK_MSK_MASTER2 | SLOT_TASK_MSK_MASTER3 )

	#define 		SLOT_TASK_MSK_SLAVE								( SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1	|  SLOT_TASK_MSK_SLAVE2	)

	#define 		SLOT_TASK_MSK_CONN								( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 |  \
																	  SLOT_TASK_MSK_MASTER2 | SLOT_TASK_MSK_MASTER3 | \
																	  SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1	|  SLOT_TASK_MSK_SLAVE2 )
#elif (CONN_MAX_NUM_CONFIG == CONN_MAX_NUM_M4_S0)

	#define			SLOT_TASK_MSK_MASTER0							BIT(0)
	#define			SLOT_TASK_MSK_MASTER1							BIT(1)
	#define			SLOT_TASK_MSK_MASTER2							BIT(2)
	#define			SLOT_TASK_MSK_MASTER3							BIT(3)


	#define 		SLOT_TASK_MSK_MASTER							( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 |  \
																	  SLOT_TASK_MSK_MASTER2 | SLOT_TASK_MSK_MASTER3 )

	#define 		SLOT_TASK_MSK_SLAVE								0  //todo

	#define 		SLOT_TASK_MSK_CONN								( SLOT_TASK_MSK_MASTER  | SLOT_TASK_MSK_SLAVE )

#elif (CONN_MAX_NUM_CONFIG == CONN_MAX_NUM_M0_S3)

	#define			SLOT_TASK_MSK_MASTER0							BIT(0)


	#define 		SLOT_TASK_MSK_MASTER							SLOT_TASK_MSK_MASTER0

	//TODO
	#define			SLOT_TASK_MSK_SLAVE0							BIT(4)
	#define			SLOT_TASK_MSK_SLAVE1							BIT(5)
	#define			SLOT_TASK_MSK_SLAVE2							BIT(6)

	#define 		SLOT_TASK_MSK_SLAVE								(SLOT_TASK_MSK_SLAVE0 | SLOT_TASK_MSK_SLAVE1 | SLOT_TASK_MSK_SLAVE2)

	#define 		SLOT_TASK_MSK_CONN								SLOT_TASK_MSK_SLAVE

#elif (CONN_MAX_NUM_CONFIG == CONN_MAX_NUM_M12_S2)

	#define			SLOT_TASK_MSK_MASTER0							BIT(0)
	#define			SLOT_TASK_MSK_MASTER1							BIT(1)
	#define			SLOT_TASK_MSK_MASTER2							BIT(2)
	#define			SLOT_TASK_MSK_MASTER3							BIT(3)
	#define			SLOT_TASK_MSK_MASTER4							BIT(4)
	#define			SLOT_TASK_MSK_MASTER5							BIT(5)
	#define			SLOT_TASK_MSK_MASTER6							BIT(6)
	#define			SLOT_TASK_MSK_MASTER7							BIT(7)
	#define			SLOT_TASK_MSK_MASTER8							BIT(8)
	#define			SLOT_TASK_MSK_MASTER9							BIT(9)
	#define			SLOT_TASK_MSK_MASTER10							BIT(10)
	#define			SLOT_TASK_MSK_MASTER11							BIT(11)


	#define			SLOT_TASK_MSK_SLAVE0							BIT(12)
	#define			SLOT_TASK_MSK_SLAVE1							BIT(13)
	#define			SLOT_TASK_MSK_SLAVE2							BIT(14)  ///support 3 slave


	#define 		SLOT_TASK_MSK_MASTER		( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 | SLOT_TASK_MSK_MASTER2 | SLOT_TASK_MSK_MASTER3 |\
												  SLOT_TASK_MSK_MASTER4 | SLOT_TASK_MSK_MASTER5 | SLOT_TASK_MSK_MASTER6 | SLOT_TASK_MSK_MASTER7 |\
												  SLOT_TASK_MSK_MASTER8 | SLOT_TASK_MSK_MASTER9 | SLOT_TASK_MSK_MASTER10 | SLOT_TASK_MSK_MASTER11 )

	#define 		SLOT_TASK_MSK_SLAVE			( SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1	|  SLOT_TASK_MSK_SLAVE2	)

	#define 		SLOT_TASK_MSK_CONN			( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 | SLOT_TASK_MSK_MASTER2 | SLOT_TASK_MSK_MASTER3 |\
												  SLOT_TASK_MSK_MASTER4 | SLOT_TASK_MSK_MASTER5 | SLOT_TASK_MSK_MASTER6 | SLOT_TASK_MSK_MASTER7 |\
												  SLOT_TASK_MSK_MASTER8 | SLOT_TASK_MSK_MASTER9 | SLOT_TASK_MSK_MASTER10 | SLOT_TASK_MSK_MASTER11 |\
												  SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1	|  SLOT_TASK_MSK_SLAVE2 )

#elif (CONN_MAX_NUM_CONFIG == CONN_MAX_NUM_M16_S3)

	#define			SLOT_TASK_MSK_MASTER0							BIT(0)
	#define			SLOT_TASK_MSK_MASTER1							BIT(1)
	#define			SLOT_TASK_MSK_MASTER2							BIT(2)
	#define			SLOT_TASK_MSK_MASTER3							BIT(3)
	#define			SLOT_TASK_MSK_MASTER4							BIT(4)
	#define			SLOT_TASK_MSK_MASTER5							BIT(5)
	#define			SLOT_TASK_MSK_MASTER6							BIT(6)
	#define			SLOT_TASK_MSK_MASTER7							BIT(7)
	#define			SLOT_TASK_MSK_MASTER8							BIT(8)
	#define			SLOT_TASK_MSK_MASTER9							BIT(9)
	#define			SLOT_TASK_MSK_MASTER10							BIT(10)
	#define			SLOT_TASK_MSK_MASTER11							BIT(11)
	#define			SLOT_TASK_MSK_MASTER12							BIT(12)
	#define			SLOT_TASK_MSK_MASTER13							BIT(13)
	#define			SLOT_TASK_MSK_MASTER14							BIT(14)
	#define			SLOT_TASK_MSK_MASTER15							BIT(15)  ///now support 16 master


	#define			SLOT_TASK_MSK_SLAVE0							BIT(16)
	#define			SLOT_TASK_MSK_SLAVE1							BIT(17)
	#define			SLOT_TASK_MSK_SLAVE2							BIT(18)  ///support 3 slave


	#define 		SLOT_TASK_MSK_MASTER		( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 | SLOT_TASK_MSK_MASTER2 | SLOT_TASK_MSK_MASTER3 |\
												  SLOT_TASK_MSK_MASTER4 | SLOT_TASK_MSK_MASTER5 | SLOT_TASK_MSK_MASTER6 | SLOT_TASK_MSK_MASTER7 |\
												  SLOT_TASK_MSK_MASTER8 | SLOT_TASK_MSK_MASTER9 | SLOT_TASK_MSK_MASTER10 | SLOT_TASK_MSK_MASTER11 |\
												  SLOT_TASK_MSK_MASTER12 | SLOT_TASK_MSK_MASTER13 | SLOT_TASK_MSK_MASTER14 | SLOT_TASK_MSK_MASTER15 )

	#define 		SLOT_TASK_MSK_SLAVE			( SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1	|  SLOT_TASK_MSK_SLAVE2	)

	#define 		SLOT_TASK_MSK_CONN			( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 | SLOT_TASK_MSK_MASTER2 | SLOT_TASK_MSK_MASTER3 |\
												  SLOT_TASK_MSK_MASTER4 | SLOT_TASK_MSK_MASTER5 | SLOT_TASK_MSK_MASTER6 | SLOT_TASK_MSK_MASTER7 |\
												  SLOT_TASK_MSK_MASTER8 | SLOT_TASK_MSK_MASTER9 | SLOT_TASK_MSK_MASTER10 | SLOT_TASK_MSK_MASTER11 |\
												  SLOT_TASK_MSK_MASTER12 | SLOT_TASK_MSK_MASTER13 | SLOT_TASK_MSK_MASTER14 | SLOT_TASK_MSK_MASTER15 |\
												  SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1	|  SLOT_TASK_MSK_SLAVE2 )
#elif (CONN_MAX_NUM_CONFIG == CONN_MAX_NUM_M24_S3)
	#define			SLOT_TASK_MSK_MASTER0							BIT(0)
	#define			SLOT_TASK_MSK_MASTER1							BIT(1)
	#define			SLOT_TASK_MSK_MASTER2							BIT(2)
	#define			SLOT_TASK_MSK_MASTER3							BIT(3)
	#define			SLOT_TASK_MSK_MASTER4							BIT(4)
	#define			SLOT_TASK_MSK_MASTER5							BIT(5)
	#define			SLOT_TASK_MSK_MASTER6							BIT(6)
	#define			SLOT_TASK_MSK_MASTER7							BIT(7)
	#define			SLOT_TASK_MSK_MASTER8							BIT(8)
	#define			SLOT_TASK_MSK_MASTER9							BIT(9)
	#define			SLOT_TASK_MSK_MASTER10							BIT(10)
	#define			SLOT_TASK_MSK_MASTER11							BIT(11)
	#define			SLOT_TASK_MSK_MASTER12							BIT(12)
	#define			SLOT_TASK_MSK_MASTER13							BIT(13)
	#define			SLOT_TASK_MSK_MASTER14							BIT(14)
	#define			SLOT_TASK_MSK_MASTER15							BIT(15)
	#define			SLOT_TASK_MSK_MASTER16							BIT(16)
	#define			SLOT_TASK_MSK_MASTER17							BIT(17)
	#define			SLOT_TASK_MSK_MASTER18							BIT(18)
	#define			SLOT_TASK_MSK_MASTER19							BIT(19)
	#define			SLOT_TASK_MSK_MASTER20							BIT(20)
	#define			SLOT_TASK_MSK_MASTER21							BIT(21)
	#define			SLOT_TASK_MSK_MASTER22							BIT(22)
	#define			SLOT_TASK_MSK_MASTER23							BIT(23) ///support 24 master



	#define			SLOT_TASK_MSK_SLAVE0							BIT(24)
	#define			SLOT_TASK_MSK_SLAVE1							BIT(25)
	#define			SLOT_TASK_MSK_SLAVE2							BIT(26)  ///support 3 slave


	#define 		SLOT_TASK_MSK_MASTER		( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 | SLOT_TASK_MSK_MASTER2 | SLOT_TASK_MSK_MASTER3 |\
												  SLOT_TASK_MSK_MASTER4 | SLOT_TASK_MSK_MASTER5 | SLOT_TASK_MSK_MASTER6 | SLOT_TASK_MSK_MASTER7 |\
												  SLOT_TASK_MSK_MASTER8 | SLOT_TASK_MSK_MASTER9 | SLOT_TASK_MSK_MASTER10 | SLOT_TASK_MSK_MASTER11 |\
												  SLOT_TASK_MSK_MASTER12 | SLOT_TASK_MSK_MASTER13 | SLOT_TASK_MSK_MASTER14 | SLOT_TASK_MSK_MASTER15 |\
												  SLOT_TASK_MSK_MASTER16 | SLOT_TASK_MSK_MASTER17 | SLOT_TASK_MSK_MASTER18 | SLOT_TASK_MSK_MASTER19 |\
												  SLOT_TASK_MSK_MASTER20 | SLOT_TASK_MSK_MASTER21 | SLOT_TASK_MSK_MASTER22 | SLOT_TASK_MSK_MASTER23)

	#define 		SLOT_TASK_MSK_SLAVE			( SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1	|  SLOT_TASK_MSK_SLAVE2	)

	#define 		SLOT_TASK_MSK_CONN			( SLOT_TASK_MSK_MASTER0 | SLOT_TASK_MSK_MASTER1 | SLOT_TASK_MSK_MASTER2 | SLOT_TASK_MSK_MASTER3 |\
												  SLOT_TASK_MSK_MASTER4 | SLOT_TASK_MSK_MASTER5 | SLOT_TASK_MSK_MASTER6 | SLOT_TASK_MSK_MASTER7 |\
												  SLOT_TASK_MSK_MASTER8 | SLOT_TASK_MSK_MASTER9 | SLOT_TASK_MSK_MASTER10 | SLOT_TASK_MSK_MASTER11 |\
												  SLOT_TASK_MSK_MASTER12 | SLOT_TASK_MSK_MASTER13 | SLOT_TASK_MSK_MASTER14 | SLOT_TASK_MSK_MASTER15 |\
												  SLOT_TASK_MSK_MASTER16 | SLOT_TASK_MSK_MASTER17 | SLOT_TASK_MSK_MASTER18 | SLOT_TASK_MSK_MASTER19 |\
												  SLOT_TASK_MSK_MASTER20 | SLOT_TASK_MSK_MASTER21 | SLOT_TASK_MSK_MASTER22 | SLOT_TASK_MSK_MASTER23 |\
												  SLOT_TASK_MSK_SLAVE0  | SLOT_TASK_MSK_SLAVE1	|  SLOT_TASK_MSK_SLAVE2 )
#else

#endif


#if (NEW_CODE_STRUCT)
	#define			SLOT_TASK_MSK_ADV									BIT(30)
	#define			SLOT_TASK_MSK_SCAN									BIT(31)
#else
	#define			SLOT_TASK_MSK_ADV									BIT(12)
	#define			SLOT_TASK_MSK_SCAN									BIT(13)
#endif






#define			SLOT_UPDT_SLAVE_CONN_CREATE						BIT(0)
#define			SLOT_UPDT_SLAVE_CONN_UPDATE						BIT(1)
#define			SLOT_UPDT_SLAVE_SYNC_DONE						BIT(2)
#define			SLOT_UPDT_SLAVE_DYNAMIC_ADJUST					BIT(3)

#define			SLOT_UPDT_SLAVE_CONNUPDATE_FAIL					BIT(4)
#define			SLOT_UPDT_ADV_SACN_STATE_CHANGE					BIT(5)
#define			SLOT_UPDT_MASTER_CONN							BIT(6)
#define			SLOT_UPDT_CONN_TERMINATE						BIT(7)

#define         SLOT_UPDT_INITIATING_STATE                      BIT(8) ///need to confirm whether overflow






#define			SLOT_DURATION_SET_SCAN							1

/******************************************************************************************************************************************************************
ADV data 37 bytes, Scan_Rsp 16 bytes, 32M clock,  all code running in SRAM, no Scan_Req/Conn_Req, cost 2353 uS
ADV data 37 bytes, Scan_Rsp 16 bytes, 32M clock,  all code running in SRAM, no Scan_Req, Conn_Req on chn39, 2700 uS (2019.09.03 SiHui test data)

ADV data 37 bytes, Scan_Rsp 37 bytes, 32M clock,  all code running in SRAM, no Scan_Req on chn37/38,
			Scan_Req&Scan_Rsp on chn39, 2940 uS -> 2890 uS (400->375 optimize) (2019.09.17 SiHui test data)

Worst Case:
PDU 37 bytes, 32M clock,  all code running in SRAM, receive scan_req/conn_req on chn37 failed,
              receive scan_req/conn_req on chn38 failed, receive scan_req success and send scan_rsp on chn39
				this timing is about: 2890+300*2 = 3490

*****************************************************************************************************************************************************************/
//TODO, optimize if ADV channel is not 37/38/39, or if ADV data & Scan_Rsp data not longest
//TODO, if none connecTable ADV or driect_adv, duration can be smaller
#define			SLOT_DURATION_ADV								6   // 625*4 = 2500uS,  625*5= 3125 uS,  625*6= 3750 uS


#if (NEW_CODE_STRUCT)

#define			SLOT_DURATION_MASTER_COMMON						master_posOffset.durationSlotNum
#define			SLOT_DURATION_SLAVE_COMMON						SLOT_DURATION_MASTER_COMMON

#else
#define			SLOT_DURATION_SLAVE_COMMON						4
#define			SLOT_DURATION_MASTER_COMMON						3
#endif





// 150us(T_ifs) + 352us(conn_req) = 502 us,  sub some margin 22(RX IRQ delay/irq_handler delay)
// real test data: 470 uS,  beginning is "u32 tick_now = rx_tick_now = clock_time();" in irq_blms_rx
//						    ending is    "while( !(reg_rf_irq_status & FLD_RF_IRQ_TX));" in
//                          "irq_handler" to "u32 tick_now = rx_tick_now = clock_time();" is 4 uS
#define			PKT_INIT_AFTER_RX_TICK							( 480 *CLOCK_16M_SYS_TIMER_CLK_1US)


// master: 30    mS * 4 , slave:  8.75 mS/10   mS/18.75 mS  276 uS, 20190921
// master: 30    mS * 4 , slave:  7.5  mS/10   mS/18.75 mS  310 uS, 20190921
// master: 31.25 mS * 4 , slave:  7.5  mS/10   mS/18.75 mS  339 uS, 20190921
#define			SLOT_PROCESS_MAX_TICK							( 400 *CLOCK_16M_SYS_TIMER_CLK_1US)

// test data 88 uS 20191014 by SiHui, consider application layer flash read or flash write timing, need add some margin
#define			SCAN_BOUNDARY_MARGIN_COMMON_TICK				( 100 *CLOCK_16M_SYS_TIMER_CLK_1US)
#define			SCAN_BOUNDARY_MARGIN_INIT_TICK					( PKT_INIT_AFTER_RX_TICK + SLOT_PROCESS_MAX_TICK)	  // initiate timing + slot_update_rebuild_allocate running potential biggest time




#define			BOUNDARY_RX_RELOAD_TICK							( 30  *CLOCK_16M_SYS_TIMER_CLK_1US)







#define			SLOT_INDEX_ALARM_LOW							BIT(31)     //15 Days
#define			SLOT_INDEX_ALARM_HIGH							0xFFFF0000  //30 Days        //(BIT(31)|BIT(30))   //23 Days






#define			PRI_WEIGHT_MASTER_DEFAULT						2

#define			PRI_INC_SLOP_DROP								1
#define			PRI_MUL_SLOP_DROP								3

enum{
	CLOCK_16M_SYS_TIMER_CLK_312US  = 5000,   //312.5*16
	CLOCK_16M_SYS_TIMER_CLK_625US  = 10000,  //625*16
	CLOCK_16M_SYS_TIMER_CLK_1250US = 20000,  //1250*16
};


typedef enum{
	SLOT_PRIORITY_LOW			=	10,

	SLOT_PRIORITY_MID_1			=	100,

	SLOT_PRIORITY_MID_2			=	150,

	SLOT_PRIORITY_MID_3			=	200,

	SLOT_PRIORITY_HIGH_THRES	=	240,

	SLOT_PRIORITY_SUPER_HIGH	=	250,    //only for slave connection establish or connection update

	SLOT_PRIORITY_MAX			=	255,
}slot_pri_t;




typedef unsigned int slot_type ;

_attribute_aligned_(4)
typedef struct{

	u32 num_cur;
	u32	update;
	u32 task_mask;

	u32 slot_tick_irq;
	u32	slot_tick_next;


	u32	slot_start_adv;
	u32	slot_start_scan;
	u32	slot_tick_irq_real;  //used for timing calculate when initiate, see the explanations in blms_m_connect
	u32 slot_idx_irq;
	u32 slot_idx_irq_real; //used for timing calculate when initiate, see the explanations in blms_m_connect

	u32 slot_mark_adv;
	u32 slot_mark_scan;
	u32	slot_idx_next;
	u32	slot_idx_start;   // start of a new slot timing
	u32 slot_idx_end;




	u32 slot_mark_conn[BLMS_MAX_CONN_NUM];   //slot_idx_mark_conn
	u32 slot_idx_mark_btxbrx[BLMS_MAX_CONN_NUM];
	u32	slot_tick_conn[BLMS_MAX_CONN_NUM];
	u32	slot_start_conn[BLMS_MAX_CONN_NUM];

	u16	slot_interval[BLMS_MAX_CONN_NUM];
	u8	priority	 [BLMS_MAX_CONN_NUM];
	u8	pri_wgt_int	 [BLMS_MAX_CONN_NUM];  //priority weight by conn_interval
	u8	pri_wgt_set	 [BLMS_MAX_CONN_NUM];  //priority weight by upper layer(host or application) setting
	u8	pri_wgt_total[BLMS_MAX_CONN_NUM];
	u8	pri_cal	 	 [BLMS_MAX_CONN_NUM];
	u8	slot_duration[BLMS_MAX_CONN_NUM];
	u8	connUpt_inst_jump[BLMS_MAX_CONN_NUM];
	s8  slot_adjust  [BLMS_MAX_CONN_NUM];
}slot_st_t;

extern _attribute_aligned_(4)	slot_st_t	AA_bltSlot;   // AA_ just for debug

#define bltSlot	AA_bltSlot





#if 0
	_attribute_aligned_(4)
	typedef struct{
		u8	slot_duration;
		u8	slot_interval;
		u8	priority;
		u8	rsvd;

		u8	weight_1;
		u8	weight_2;
		u8	rsvd2;
		u8	rsvd3;


		u32 slot_mark_conn;
		u32 slot_start_conn;
		u32 slot_tick_conn;
	}conn_slot_t;
	extern _attribute_aligned_(4)	conn_slot_t AA_connSlot[];

	#define NEW_SLOT_VARIA_EN					0
	#define	connSlot							AA_connSlot
#endif


extern u8	AA_slot_map[];    // AA_ just for debug

#define slot_map	AA_slot_map

/******************************* Macro & Enumeration & Structure Definition for Stack End ******************************/


















/******************************* Stack Interface Begin, user can not use!!! ********************************************/

int		blt_llms_slotUpdateRebuildAllocate(void);

int		blt_llms_startSlotTiming(void);
int 	blt_llms_rebuildSlotTimingAdv(void);
int 	blt_llms_rebuildSlotTimingScan(void);
int 	blt_llms_findAdvSlotAvailable(void);

void 	blt_llms_procStateChange(void);

void 	blt_llms_procScanSlotTask (void);
void 	blt_llms_procScanInsertTask (void);
u32 	blt_llms_calculateScanDuration (u32 scan_tick_left);

u32 	blt_llms_findFirstAvailableStartSlot(u32 mark_slot, u16 slot_interval);
void 	blt_llms_procSlotIndexOverflow(void);

static inline void blt_slot_setConnSlotPriority(u8 conn_idx, u16 pri){
	if(pri > SLOT_PRIORITY_MAX){
		pri = SLOT_PRIORITY_MAX;
	}
#if NEW_SLOT_VARIA_EN
	connSlot[conn_idx].priority = pri;
#else
	bltSlot.priority[conn_idx] = pri;
#endif
}

static inline u8 blt_slot_getConnSlotPriority(u8 conn_idx){
#if NEW_SLOT_VARIA_EN
	return	connSlot[conn_idx].priority;
#else
	return bltSlot.priority[conn_idx];
#endif

}


static inline void blt_slot_clearConnSlotPriority(u8 conn_idx){
#if NEW_SLOT_VARIA_EN
	connSlot[conn_idx].priority = 0;
#else
	bltSlot.priority[conn_idx] = 0;
#endif
}

void blt_slot_increaseConnSlotPriority(u8 conn_idx, int inc);
void blt_slot_increaseConnSlotPriCal  (u8 conn_idx, int inc);
void blt_slot_decreaseConnSlotPriCal  (u8 conn_idx, int inc);

static inline void blt_slot_addTaskMask(u32 task){
	bltSlot.task_mask |= (task);
}

static inline void blt_slot_removeTaskMask(u32 task){
	bltSlot.task_mask &= ~(task);
}

static inline void blt_slot_clearTaskMask(void){
	bltSlot.task_mask = 0;
}

#if (NEW_CODE_STRUCT)
static inline void blt_slot_addUpdate(u32 updt){
	bltSlot.update |= (updt);
}

static inline void blt_slot_removeUpdate(u32 updt){
	bltSlot.update &= ~(updt);
}
#else
static inline void blt_slot_addUpdate(u8 updt){
	bltSlot.update |= (updt);
}

static inline void blt_slot_removeUpdate(u8 updt){
	bltSlot.update &= ~(updt);
}
#endif
/******************************* Stack Interface End *******************************************************************/




#endif /* LLMS_SLOT_H_ */
