/********************************************************************************************************
 * @file     smp.h
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     Jun. 17, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#ifndef BLE_SMP_H_
#define BLE_SMP_H_

#include "stack/ble/ble_common.h"
#include "stack/ble/smp/smp.h"
#include "stack/ble/smp/smp_const.h"
#include "stack/ble/llms/llms_device.h"
#include "stack/ble/llms/llms_slot.h"


/******************************* Macro & Enumeration & Structure Definition for Stack Begin, user can not use!!!!  *****/

//TODO:  all debug MACRO should remove at last
#define			DBG_SMP_ERR_EN				0
//TODO: delete all debug codes after functions are tested OK
#define 		SMP_ERR_DEBUG(x)			do{irq_disable();write_reg32(0x40000, (x));while(1);}while(0)
//#define 		SMP_ERR_DEBUG(x)

#define SMP_INFO_STORAGE_IN_FLASH			1
#define SMP_INFO_STORAGE_IN_EEPROM			2
#define SMP_INFO_STORAGE_OTHER_MCU	 		3

#define PAIRING_PHASE_IDLE   				0x00UL
#define PAIRING_PHASE_1_OK   				0x01UL
#define PAIRING_PHASE_2_ENC  				0x02UL
#define PAIRING_PHASE_2_OK   				0x03UL

//define smp database(keys/peer device informations ...)
#ifndef SMP_DATABASE_INFO_SOURCE
#define SMP_DATABASE_INFO_SOURCE			SMP_INFO_STORAGE_IN_FLASH
#endif


#define SMP_STANDARD_PAIR   				0
#define SMP_FAST_CONNECT   					1


#define  ADDR_NOT_BONDED					0xFF
#define  ADDR_NEW_BONDED					0xFE
#define  ADDR_DELETE_BOND					0xFD

#define  KEY_FLAG_IDLE						0xFF
#define  KEY_FLAG_NEW						0xFE
#define  KEY_FLAG_FAIL						0xFD

#define  FlAG_BOND							BIT(0)
#define  FLAG_FASTSMP						BIT(4)

#define  SLAVE_TRIGGER_SMP_FIRST_PAIRING	0   	//first pair, slave send security_request to trigger master's pairing&encryption
#define  MASTER_TRIGGER_SMP_FIRST_PAIRING	BIT(0)

#define  SLAVE_TRIGGER_SMP_AUTO_CONNECT		0   	//auto connect, slave send security_request to trigger master's encryption
#define  MASTER_TRIGGER_SMP_AUTO_CONNECT	BIT(1)


typedef struct{
	u8		type;				//RFU(3)_MD(1)_SN(1)_NESN(1)-LLID(2)
	u8  	rf_len;				//LEN(5)_RFU(3)
	u16		l2capLen;
	u16		chanId;
	u8  	opcode;
	u8 		data[21];
}smp2llcap_type_t;

extern smp2llcap_type_t smpResSignalPkt;

typedef struct{
	u8  	secReq_conn;    //BIT<7:4> reConn;  BIT<3:0> newConn;
	u8  	secReq_pending;
	u16 	pending_ms;
}secReq_ctl_t;

extern _attribute_aligned_(4) secReq_ctl_t	blc_SecReq_ctrl;

typedef struct{
	u8  	trigger_mask;
	u8  	smp_begin_flg;
	u8  	manual_smp_start;
	u8  	smp_start_pending;
}smp_m_trig_t;  //master SMP trigger data type

extern _attribute_aligned_(4) smp_m_trig_t	blt_smpTrig;

/*
 * smp parameter about peer device.
 * Attention:The use of pointers in an array of structures requires attention to prevent the
 * reference to the corresponding structure member variable addresses from being misaligned.
 */
_attribute_aligned_(4)
typedef struct{
	u8		rsvd1;   // use smp own param's, removed here. bond and key_size
	u8 		peer_addr_type;  //address used in link layer connection
	u8		peer_conn_addr[6];

	u8      rsvd2;
	u8		peer_id_address_type;  //peer identity address information in key distribution, used to identify
	u8		peer_id_address[6];

	u8		peer_pairing_rand[16];

	u8		peer_confirm[16];

//	u8		peer_csrk[16];
	u8		peer_irk[16];
	u8  	peer_ltk[16];   //peer device LTK

	u8		peer_random[8]; //peer device RANDOM

	u16 	peer_ediv;      //peer device EDIV
	u16     rsvd3;          //must be aligned
}smp_param_peer_t;


_attribute_aligned_(4)
typedef struct{

	u8 						tk_status;
	u8 						pairing_busy;
	u8 						key_distribute;  //flag key distribute
	u8 						bonding_enable;

	u8 						save_key_flag;
	u8 						smpDistirbuteKeyOrder;//record key distribute order
	smp_keyDistribution_t  	smp_DistributeKeySend; //send key transmit
	smp_keyDistribution_t  	smp_DistributeKeyRecv; //receive key transmit

	u32 					smp_phase_chk;

	u32 					smp_timeout_start_tick;
}smp_st_t;  //SMP status data type


/*
 * smp parameter about own device.
 * Attention:The use of pointers in an array of structures requires attention to prevent the
 * reference to the corresponding structure member variable addresses from being misaligned.
 */
_attribute_aligned_(4)
typedef struct{
	u8 						sc_pairing; // When pairing for the 1st time, whether using SC or legacy pairing method
	u8 						stk_method;  // final available stk generate method
	u8 						sc_passkey_cnt;//20 timer passkey entry for security connection
	u8 						peerKey_mask;// double check peer key distribute order


	u8						paring_tk[16];   // in security connection to keep own random
	u8						own_ltk[16];    //used for generate ediv and random

	u8 						own_rand[16];
//	u8 						own_confirm[16];
	u8						own_csrk[16];
	u8						own_irk[16];

	smp_authReq_t			auth_req;    // 1B
	u8						own_addr_type;  // 1B current connection peer own type
	u8						own_conn_addr[6];

	u8						encryt_key_size;//min(max key size of master, max key size of slave).
	smp_paring_req_rsp_t  	pairing_req; // 7B

	smp_paring_req_rsp_t  	pairing_rsp; // 7B
	u8                      rsvd; //must be aligned

 #if (SECURE_CONNECTION_ENABLE)
	u8                      macKey[16];
 #endif

}smp_param_own_t;


_attribute_aligned_(4)
typedef struct {
	u8 		security_level;
	u8 		bonding_mode;
	u8 		IO_capability;
	u8 		ecdh_debug_mode;//1:debug_mode,0:ecdh public/private key pairs distribute

	u8 		MITM_protetion;
	u8 		oob_enable;
	u8 		pairing_method;  //paring_method
	u8 		keyPress_en;

} smp_prop_t; //SMP property parameters ConFiged by user (slave can separate to multi_mac device )


extern smp_prop_t blt_smpProp[];
extern smp_prop_t *blms_p_prop;

extern _attribute_aligned_(4) 	smp_param_own_t		smp_param_own[];
extern smp_param_own_t  *blms_p_own;
extern _attribute_aligned_(4)	smp_param_peer_t	smp_param_peer[];
extern smp_param_peer_t	*blms_p_peer;

extern _attribute_aligned_(4)	smp_st_t			smp_sts_param[];
extern smp_st_t	*blms_p_sts;

extern u8 	smpParingOwnIRK[16];
extern u8 	smpParingOwnCSRK[16];
extern u8 	smpOwnParingConfirm[16];

typedef int (*smp_init_handler_t)(u16 conn, u8 *p);
extern smp_init_handler_t	func_smp_init;

#if (SECURE_CONNECTION_ENABLE)
extern const u8	PublicKey[64];
extern const u8 PrivateKey[32];
extern _attribute_aligned_(4)	smp_sc_key_t	smp_sc_key;
typedef u8 * (*smp_sc_cmd_handler_t)(u16 conn, u8*p);
typedef void (*smp_sc_pushPkt_handler_t)(u16 conn, u8 type);
extern smp_sc_cmd_handler_t		func_smp_sc_proc;
extern smp_sc_pushPkt_handler_t	func_smp_sc_pushPkt_proc;
#endif

/******************************* Macro & Enumeration & Structure Definition for Stack End ******************************/




/******************************* Macro & Enumeration variables for User Begin ******************************************/
typedef enum {
	non_debug_mode = 0,  // ECDH distribute private/public key pairs
	debug_mode = 1,      // ECDH use debug mode private/public key pairs
} ecdh_keys_mode_t;

// "SecReq" refer to "security request"
typedef enum {
	SecReq_NOT_SEND = 0,   // do not send "security request" after link layer connection established
	SecReq_IMM_SEND = BIT(0),   //"IMM" refer to immediate, send "security request" immediately after link layer connection established
	SecReq_PEND_SEND = BIT(1),  //"PEND" refer to pending,  pending "security request" for some time after link layer connection established, when pending time arrived. send it
}secReq_cfg;

// "PairReq" refer to "pairing request"
typedef enum {
	PairReq_SEND_upon_SecReq = 0,   // master send "pairing request" when received slave's "security request"
	PairReq_AUTO_SEND		 = 1,   // master send "pairing request" automatically, regardless of "security request"
}PairReq_cfg;

/******************************* Macro & Enumeration variables for User End ********************************************/





/******************************* Stack Interface Begin, user can not use!!! ********************************************/
void 	blt_smp_param_pre_init(void);
int 	blt_smp_param_init (void);
u8* 	blt_smp_pushParingFailed(u8 failReason);
int 	blt_smp_llEncryptionDone(u16 connHandle, u8 status, u8 enc_enable);
void 	blt_smp_setParingBusy(u16 connHandle, u8 busy);
int 	blt_smp_sendSecurityRequest (u16 connHandle);
void 	blt_smp_procSlaveSecurityRequest(u16 connHandle);
void    blt_smp_procMasterPairingRequest(u16 connHandle);
void 	blt_smp_proc_paring_loop(u16 connHandle);
u8 		blt_smp_isSmpBusy(void);  //TODO
int 	bls_smp_llGetLtkReq(u16 connHandle, u8* random, u16 ediv);
u8 *	blt_smp_pushSmpCmdPkt(u16 connHandle, u8 type);
void 	blt_smp_saveBondingKey(u16 connHandle);
u8 * 	blt_smp_sendInfo(u16 connHandle);
void 	blt_smp_procParingEnd(u16 connHandle, u8 err_reason);
u8 * 	blt_smp_l2capSmpCmdHandler(u16 connHandle, u8* p);
void 	blt_smp_certTimeoutLoopEvt(u16 connHandle);
paring_sts_t blt_smp_get_paring_status(u16 connHandle);  //connHandle for future mutiConnection, now use direct BLS_CONN_HANDLE is OK
void    blt_smp_encChangeEvt(u8 status, u16 connHandle, u8 enc_enable);
smp_keyDistribution_t blt_smp_setInitiatorKey (smp_param_own_t *pSmpOwn, u8 LTK_En, u8 IRK_En, u8 CSRK_En);
smp_keyDistribution_t blt_smp_setResponderKey (smp_param_own_t *pSmpOwn, u8 LTK_En, u8 IRK_En, u8 CSRK_En);
void 	blt_smp_resetSmpStateParam(u16 connHandle);
int 	blt_smp_getStkGenMethod (smp_param_own_t *blms_p_own, int SC_en);

static inline void blt_smp_setCertTimeoutTick (u16 conn_handle, u32 t)
{
	smp_sts_param[conn_handle & CONN_IDX_MASK].smp_timeout_start_tick = t;
}

#if (SECURE_CONNECTION_ENABLE)
u8* 	blt_smp_sc_handler(u16 connHandle, u8*p);
void 	blt_smp_sc_pushPkt_handler(u16 connHandle, u8 type);
#endif

/******************************* Stack Interface End *******************************************************************/




/******************************* User Interface  Begin *****************************************************************/
void 	blc_smp_smpParamInit(void);
void 	blc_smp_configSecurityRequestSending( secReq_cfg newConn_cfg,  secReq_cfg reConn_cfg, u16 pending_ms);
void    blc_smp_configPairingRequestSending( PairReq_cfg newConn_cfg,  PairReq_cfg reConn_cfg);

void 	blc_smp_setSecurityLevel(le_security_mode_level_t  mode_level);
void 	blc_smp_setSecurityLevel_master(le_security_mode_level_t  mode_level);
void 	blc_smp_setSecurityLevel_slave(le_security_mode_level_t  mode_level);

void    blc_smp_setParingMethods(paring_methods_t  method);
void 	blc_smp_setParingMethods_master(paring_methods_t  method);    //select paring methods
void 	blc_smp_setParingMethods_slave (paring_methods_t  method);    //select paring methods

void    blc_smp_setIoCapability(io_capability_t ioCapablility);
void    blc_smp_setIoCapability_master(io_capability_t ioCapablility);
void    blc_smp_setIoCapability_slave(io_capability_t ioCapablility);

void    blc_smp_enableOobAuthentication(int OOB_en);
void 	blc_smp_enableOobAuthentication_master (int OOB_en);
void 	blc_smp_enableOobAuthentication_slave (int OOB_en);

void    blc_smp_setBondingMode(bonding_mode_t mode);
void 	blc_smp_setBondingMode_master(bonding_mode_t mode);
void 	blc_smp_setBondingMode_slave(bonding_mode_t mode);

void    blc_smp_enableAuthMITM(int MITM_en);
void 	blc_smp_enableAuthMITM_master(int MITM_en);
void 	blc_smp_enableAuthMITM_slave(int MITM_en);

void    blc_smp_enableKeypress(int keyPress_en);
void 	blc_smp_enableKeypress_master(int keyPress_en);
void 	blc_smp_enableKeypress_slave(int keyPress_en);

void    blc_smp_setEcdhDebugMode(ecdh_keys_mode_t mode);
void 	blc_smp_setEcdhDebugMode_master(ecdh_keys_mode_t mode);
void 	blc_smp_setEcdhDebugMode_slave(ecdh_keys_mode_t mode);

void 	blc_smp_setSecurityParamters(bonding_mode_t mode, int MITM_en, paring_methods_t method, int OOB_en,
		                             int keyPress_en, io_capability_t ioCapablility);
void 	blc_smp_setSecurityParamters_master(bonding_mode_t  bond_mode, int MITM_en, paring_methods_t method, int OOB_en,
		                                   int keyPress_en,io_capability_t ioCapablility);
void 	blc_smp_setSecurityParamters_slave(bonding_mode_t  bond_mode, int MITM_en, paring_methods_t method, int OOB_en,
		                                   int keyPress_en,io_capability_t ioCapablility);

u8    	blc_smp_setTK_by_OOB              (u16 connHandle, u8 *oobData);      //used for set oob data
u8      blc_smp_isWaitingToSetPasskeyEntry(u16 connHandle);
u8 		blc_smp_isWaitingToCfmNumericComparison(u16 connHandle);
u8    	blc_smp_setTK_by_PasskeyEntry     (u16 connHandle, u32 pinCodeInput);
void    blc_smp_setNumericComparisonResult(u16 connHandle, bool YES_or_NO);   //numeric compare confirm, 1: YES  0: NO
int 	blc_smp_isParingBusy(u16 connHandle);

#if (BLMS_MULTI_LOCAL_MAC_EN)
void 	blc_smp_setSecurityLevel_slave_by_device_index(int slave_dev_ind, le_security_mode_level_t  mode_level);
void 	blc_smp_setParingMethods_slave_by_device_index(int slave_dev_ind, paring_methods_t  method);
void 	blc_smp_setIoCapability_slave_by_device_index(int slave_dev_ind, io_capability_t ioCapablility);
void 	blc_smp_enableOobAuthentication_slave_by_device_index(int slave_dev_ind, int OOB_en);
void 	blc_smp_setBondingMode_slave_by_device_index(int slave_dev_ind, bonding_mode_t mode);
void 	blc_smp_enableAuthMITM_slave_by_device_index(int slave_dev_ind, int MITM_en);
void 	blc_smp_enableKeypress_slave_by_device_index(int slave_dev_ind, int keyPress_en);
void 	blc_smp_setEcdhDebugMode_slave_by_device_index(int slave_dev_ind, ecdh_keys_mode_t mode);
void 	blc_smp_setSecurityParamters_slave_by_device_index(int slave_dev_ind, bonding_mode_t  bond_mode, int MITM_en, paring_methods_t method,
											               int OOB_en, int keyPress_en,io_capability_t ioCapablility);
#endif
/******************************* User Interface  End  ******************************************************************/


#endif /* BLE_SMP_H_ */

