/********************************************************************************************************
 * @file	 app_device.h
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     May. 22, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#ifndef APP_DEVICE_H_
#define APP_DEVICE_H_

#define PEER_SLAVE_USE_RPA_EN						0	//peer slave device use RPA(resolvable private address)



#define DEVICE_CHAR_INFO_MAX_NUM		(MASTER_MAX_NUM + SLAVE_MAX_NUM)    //4 master, 3 slave most



#define CHAR_HANDLE_MAX					8
/***************** connection character device information ****************************
 *
 * Demo master device char_handle(ATT handle) define as follows, assuming that peer device(BLE slave) is TELINK HID device
 *   char_handle[0] :  MIC
 *   char_handle[1] :  Speaker
 *   char_handle[2] :  OTA
 *   char_handle[3] :  Consume Report In
 *   char_handle[4] :  Key Report In
 *   char_handle[5] :
 *   char_handle[6] :  BLE Module, SPP Server to Client
 *   char_handle[7] :  BLE Module, SPP Client to Server
 *************************************************************************************/

typedef struct {
	u8 id_adrType;					//identity address type
	u8 id_addr[BLE_ADDR_LEN];		//identity address
	u8 reserved;
	u8 irk[16];
} rpa_addr_t;



//Attention: manual set 4 byte aligned
typedef struct
{
	u16 	conn_handle;
	u8 		conn_state;				// 1: connect;  0: disconnect
	u8		char_handle_valid;      // 1: peer device's attHandle is available;   0: peer device's attHandle not available

	u8		rsvd[4];  				//just for debug, 16 byte aligned, convenient to see

	u8		peer_adrType;
	u8		peer_addr[6];
	u8		peer_RPA;         //RPA: resolvable private address
	//rpa_addr_t *pPeer_RPA;    //only when peer  mac_address is RPA, this pointer is useful

#if 0   //for local multi_mac_address
	u8		local_adrType;
	u8		local_addr[6];
	u8		local_RPA;        //RPA: resolvable private address
	//rpa_addr_t *pLocal_RPA;   //only when local mac_address is RPA, this pointer is useful
#endif

	u16		char_handle[CHAR_HANDLE_MAX];

}dev_char_info_t;



extern dev_char_info_t  cur_master_conn_device;
extern dev_char_info_t	conn_dev_list[];


extern int	conn_master_num;
extern int	conn_slave_num;

/*
 * Used for add device information to conn_dev_list.
 */
int 	dev_char_info_insert (dev_char_info_t* dev_char_info);


/*
 * Used for add peer device service ATThandle.
 */
int 	dev_char_info_add_peer_att_handle (dev_char_info_t* dev_char_info);


/*
 * Used for delete device information from conn_dev_list by connHandle
 */
int 	dev_char_info_delete_by_connhandle (u16 connhandle);

/*
 * Used for delete device information from conn_dev_list by peer mac_address
 */
int 	dev_char_info_delete_by_peer_mac_address (u8 adr_type, u8* addr);



dev_char_info_t* 	dev_char_info_search_by_peer_mac_address (u8 adr_type, u8* addr);
dev_char_info_t* 	dev_char_info_search_by_connhandle (u16 connhandle);



int 	dev_char_info_delete_by_local_mac (u8 adr_type, u8* addr);
dev_char_info_t* 	dev_char_info_search_by_local_mac (u8 adr_type, u8* addr);


/*
 * Used for judge if current device conn_handle
 */
bool	dev_char_info_is_connection_state_by_conn_handle(u16 connhandle);



#if 0
static inline u16 	dev_char_info_get_conn_handle_by_slave_index (int slave_index)
{
	return conn_dev_list[slave_index].conn_handle;
}


static inline u16 	dev_char_info_get_conn_handle_by_master_index (int master_index)
{
	return conn_dev_list[master_index].conn_handle;
}
#endif












#if (BLE_MASTER_SIMPLE_SDP_ENABLE)

#define ATT_BOND_MARK 		0x5A
#define ATT_ERASE_MARK		0x00


typedef struct{
	u8	flag;
	u8	adr_type;
	u8	addr[6];

	u8	rsvd[8];  //very important: 16 byte aligned, to avoid different flash page write for a sequence data

#if (PEER_SLAVE_USE_RPA_EN)
	u8  irk[16];   //TODO: if peer device mac_adress is RPA(resolvable private address), IRK will be used
#endif

	u16	char_handle[CHAR_HANDLE_MAX];
}dev_att_t;



int		dev_char_info_store_peer_att_handle(dev_char_info_t*);
int		dev_char_info_search_peer_att_handle_by_peer_mac(u8 adr_type, u8* addr, dev_att_t *);
int		dev_char_info_delete_peer_att_handle_by_peer_mac(u8 addrType, u8 *addr);
#endif












#endif /* APP_DEVICE_H_ */
