/********************************************************************************************************
 * @file	 app_device.c
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     May. 22, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"


#include "app_att.h"
#include "app_device.h"
#include "app_ui.h"
#include "custom_pair.h"








/*
 * Used for store information of connected devices.
 *
 * 0 ~ (MASTER_MAX_NUM - 1) is for master,  MASTER_MAX_NUM ~ (MASTER_MAX_NUM + SLAVE_MAX_NUM - 1) s for slave
 *
 * e.g.    MASTER_MAX_NUM	SLAVE_MAX_NUM			master					  slave
 *                1				  1				conn_dev_list[0]		conn_dev_list[1]
 *                2				  2				conn_dev_list[0..1]		conn_dev_list[2..3]
 *                3				  2				conn_dev_list[0..2]		conn_dev_list[3..4]
 *                4				  3				conn_dev_list[0..3]		conn_dev_list[4..6]
 */
dev_char_info_t	conn_dev_list[DEVICE_CHAR_INFO_MAX_NUM];

/*
 * Special Design: when a device connection terminate, e.g. device index is 1, we will not only remove conn_dev_list[1],
 * 				   but also copy conn_dev_list[2] (assume that this device is in connection state) to conn_dev_list[1],
 * 				   			copy conn_dev_list[3] (assume that this device is in connection state) to conn_dev_list[2],
 * 				   			......
 *
 * So for 4 master 3 slave demo, the relationship between device in connection state and conn_master_num/conn_slave_num is as follows��
 *
 * 			conn_master_num	   					device in connection state
 *
 * 				   1		  						conn_dev_list[0]
 * 				   2								conn_dev_list[0]/conn_dev_list[1]
 * 				   3								conn_dev_list[0]/conn_dev_list[1]/conn_dev_list[2]
 * 				   4								conn_dev_list[0]/conn_dev_list[1]/conn_dev_list[2]/conn_dev_list[3]
 *
 * 			conn_slave_num
 *
 * 				   1		  						conn_dev_list[4]
 * 				   2								conn_dev_list[4]/conn_dev_list[5]
 * 				   3								conn_dev_list[4]/conn_dev_list[5]/conn_dev_list[6]
 *
 *     			.conn_state must be 1, conn_dev_list[2].conn_state
 */
int conn_master_num = 0;   //current master number in connection state
int conn_slave_num = 0;    //current slave number in connection state




dev_char_info_t cur_master_conn_device;




/*
 * Used for add device information to conn_dev_list.
 */
int dev_char_info_insert (dev_char_info_t* dev_char_info)
{
	int index = 100;
	if( dev_char_info->conn_handle & BLM_CONN_HANDLE ){  //master
		if(conn_master_num < MASTER_MAX_NUM){
			index = conn_master_num;
			conn_master_num ++;
		}
	}
	else if( dev_char_info->conn_handle & BLS_CONN_HANDLE ){  //slave
		if(conn_slave_num < SLAVE_MAX_NUM){
			index = MASTER_MAX_NUM + conn_slave_num;
			conn_slave_num ++;
		}
	}

	if(index < MASTER_MAX_NUM + SLAVE_MAX_NUM){
		memcpy(&conn_dev_list[index], dev_char_info, sizeof(dev_char_info_t));
		conn_dev_list[index].conn_state = 1;

		return 0;   //success
	}
	else{
		return 1;   //fail
	}

}





/*
 * Used for add peer device service ATThandle.
 */
int dev_char_info_add_peer_att_handle (dev_char_info_t* dev_char_info)
{
	int i;
	for(i=0; i< conn_master_num; i++){
		if( conn_dev_list[i].conn_handle == dev_char_info->conn_handle){
			break;
		}
	}

	if( i < conn_master_num){
		for(int j=0; j<CHAR_HANDLE_MAX;  j++){
			conn_dev_list[i].char_handle[j] = dev_char_info->char_handle[j];
		}

		conn_dev_list[i].char_handle_valid = 1;

		return 0;   //success
	}
	else{
		return 1;   //fail
	}
}




/*
 * Used for delete device information from conn_dev_list by connHandle
 */
int dev_char_info_delete_by_connhandle (u16 connhandle)
{
	foreach (i, DEVICE_CHAR_INFO_MAX_NUM)
	{
		if(conn_dev_list[i].conn_handle == connhandle && conn_dev_list[i].conn_state)  //match
		{
//			memset(&conn_dev_list[i], 0, sizeof(dev_char_info_t));

			if(connhandle & BLM_CONN_HANDLE){   //master
				memcpy (&conn_dev_list[i], &conn_dev_list[i + 1], (conn_master_num - i - 1) * sizeof (dev_char_info_t));
				conn_master_num --;
				memset(&conn_dev_list[conn_master_num], 0, sizeof(dev_char_info_t));
			}
			else{  //slave
				memcpy (&conn_dev_list[i], &conn_dev_list[i + 1], (conn_slave_num + MASTER_MAX_NUM - i - 1) * sizeof (dev_char_info_t));
				conn_slave_num --;
				memset(&conn_dev_list[conn_slave_num + MASTER_MAX_NUM], 0, sizeof(dev_char_info_t));
			}

			return 0;
		}
	}
	return 1;  //no find the peer device address.
}


/*
 * Used for delete device information from conn_dev_list by peer device's mac address
 */
int 	dev_char_info_delete_by_peer_mac_address (u8 adr_type, u8* addr)
{
	foreach (i, DEVICE_CHAR_INFO_MAX_NUM)
	{
		if(conn_dev_list[i].conn_state)
		{
			int mac_match = 0;
			if( IS_RESOLVABLE_PRIVATE_ADDR(adr_type, addr) ){
				//TODO
			}
			else{
				if(adr_type == conn_dev_list[i].peer_adrType && (!memcmp (addr, conn_dev_list[i].peer_addr, 6)) ){
					mac_match = 1;
				}
			}

			u16 connhandle = conn_dev_list[i].conn_handle;

			if(mac_match){
				if(connhandle & BLM_CONN_HANDLE){   //master
					memcpy (&conn_dev_list[i], &conn_dev_list[i + 1], (conn_master_num - i - 1) * sizeof (dev_char_info_t));
					memset(&conn_dev_list[conn_master_num], 0, sizeof(dev_char_info_t));
					conn_master_num --;
				}
				else{  //slave
					memcpy (&conn_dev_list[i], &conn_dev_list[i + 1], (conn_slave_num + MASTER_MAX_NUM - i - 1) * sizeof (dev_char_info_t));
					memset(&conn_dev_list[conn_slave_num + MASTER_MAX_NUM], 0, sizeof(dev_char_info_t));
					conn_slave_num --;
				}

				return 0;
			}

		}
	}

	return 1;  //no find the peer device address.
}







/*
 * Used for search device information from conn_dev_list by connHandle.
 * return  - 0 no peer device.
 * 			 others - device info
 */
dev_char_info_t* dev_char_info_search_by_connhandle (u16 connhandle)
{
	foreach (i, DEVICE_CHAR_INFO_MAX_NUM)
	{
		if(conn_dev_list[i].conn_handle == connhandle && conn_dev_list[i].conn_state)
		{
			return &conn_dev_list[i];   // find the peer device
		}
	}

	return 0;  // no find the peer device
}




/*
 * Used for search device information from conn_dev_list by peer device mac_address
 */
dev_char_info_t* dev_char_info_search_by_peer_mac_address (u8 adr_type, u8* addr)
{
	foreach (i, DEVICE_CHAR_INFO_MAX_NUM)
	{

		int mac_match = 0;
		if( IS_RESOLVABLE_PRIVATE_ADDR(adr_type, addr) ){
			//TODO
		}
		else{
			if(adr_type == conn_dev_list[i].peer_adrType && (!memcmp (addr, conn_dev_list[i].peer_addr, 6)) ){
				mac_match = 1;
			}
		}

		if(mac_match){
			return &conn_dev_list[i];   // find the peer device
		}
	}

	return 0;  // no find the peer device
}








#if (BLE_MASTER_SIMPLE_SDP_ENABLE)

int		dev_char_info_store_peer_att_handle(dev_char_info_t* pdev_char)
{
	u8 mark;
	u32 current_flash_adr;
	for(current_flash_adr = FLASH_SDP_ATT_ADRRESS; current_flash_adr < (FLASH_SDP_ATT_ADRRESS + FLASH_SDP_ATT_MAX_SIZE); current_flash_adr += sizeof(dev_att_t) )
	{
		mark = *(u8 *)current_flash_adr;

		if( mark == U8_MAX){

			flash_write_page( current_flash_adr + OFFSETOF(dev_att_t, adr_type),  7, (u8 *)&pdev_char->peer_adrType);  // peer_adrType(1)+peer_addr(6)

#if (PEER_SLAVE_USE_RPA_EN)
			if( IS_RESOLVABLE_PRIVATE_ADDR(pdev_char->peer_adrType, pdev_char->peer_addr) ){
				//TODO, store irk to flash
			}
#endif

//			 char_handle[0] :  MIC
//			 char_handle[1] :  Speaker
//			 char_handle[2] :  OTA
//			 char_handle[3] :  Consume Report
//			 char_handle[4] :  Key Report
//			 char_handle[5] :
//			 char_handle[6] :  BLE Module, SPP Server to Client
//			 char_handle[7] :  BLE Module, SPP Client to Server
			#if(UI_AUDIO_ENABLE)
				flash_write_page( current_flash_adr + OFFSETOF(dev_att_t, char_handle) + 0*2,  2, (u8 *)&pdev_char->char_handle[0]);   //save MIC att_handle
			#endif
			flash_write_page( current_flash_adr + OFFSETOF(dev_att_t, char_handle) + 2*2,  2, (u8 *)&pdev_char->char_handle[2]);   //save OTA att_handle
			flash_write_page( current_flash_adr + OFFSETOF(dev_att_t, char_handle) + 3*2,  2, (u8 *)&pdev_char->char_handle[3]);   //save Consume Report att_handle
			flash_write_page( current_flash_adr + OFFSETOF(dev_att_t, char_handle) + 4*2,  2, (u8 *)&pdev_char->char_handle[4]);   //save Key Report att_handle

			mark = ATT_BOND_MARK;
			flash_write_page( current_flash_adr,  1, (u8 *)&mark);

			return current_flash_adr;  //Store Success
		}
	}






	return 0;  //Store Fail
}



int		dev_char_info_search_peer_att_handle_by_peer_mac(u8 adr_type, u8* addr, dev_att_t* pdev_att)
{


	u8 mark;
	u32 current_flash_adr;
	for(current_flash_adr = FLASH_SDP_ATT_ADRRESS; current_flash_adr < (FLASH_SDP_ATT_ADRRESS + FLASH_SDP_ATT_MAX_SIZE); current_flash_adr += sizeof(dev_att_t) )
	{
		mark = *(u8 *)current_flash_adr;


		if(mark == U8_MAX){
			return 0;   //Search Fail
		}
		else if(mark == ATT_ERASE_MARK){
			continue;   //Search for next unit
		}
		else if( mark == ATT_BOND_MARK)
		{
			memcpy( (u8 *)pdev_att, (u8 *)current_flash_adr  , sizeof(dev_att_t) );  //same effect as: flash_read_page(current_flash_adr, sizeof(dev_att_t), (u8 *)pdev_att);


			int addr_match = 0;
#if (PEER_SLAVE_USE_RPA_EN)
			if( IS_RESOLVABLE_PRIVATE_ADDR(pdev_att->adr_type, pdev_att->addr) ){
				//TODO, resolve address by irk
				if(0){
					addr_match = 1;
				}

			}
			else
#endif
			{
				if( adr_type == pdev_att->adr_type && !memcmp(addr, pdev_att->addr, 6 )){  //match
					addr_match = 1;
				}
			}

			if(addr_match){
				return current_flash_adr;
			}


		}


	}

	return 0;   //Search Fail

}

int dev_char_info_delete_peer_att_handle_by_peer_mac(u8 addrType, u8 *addr)
{
	dev_att_t dev_info;

	for(u32 cur_flash_addr = FLASH_SDP_ATT_ADRRESS;
		cur_flash_addr < FLASH_SDP_ATT_ADRRESS + FLASH_SDP_ATT_MAX_SIZE;
		cur_flash_addr += sizeof(dev_att_t))
	{
		u8 flag = *(u8*)cur_flash_addr;

		//have no device information
		if(flag == 0xff)
			return 1;//not find

		if(flag == ATT_BOND_MARK){
			//only read per device MAC address type and MAC address
			memcpy((u8*)&dev_info, (u8*)cur_flash_addr, 8);//same effect as:flash_read_page(cur_flash_addr, 8, (u8*)&dev_info);
		#if(PEER_SLAVE_USE_RPA_EN)
			if(IS_RESOLVABLE_PRIVATE_ADDR(addrType, addr)){
				//todo: resolve private address using IRK
			}
			else
		#endif
			{
				if(dev_info.adr_type == addrType && !memcmp(dev_info.addr, addr, 6)){
					u8 temp = ATT_ERASE_MARK;
					flash_write_page(cur_flash_addr, 1, (u8*)&temp);
					return 0;//find
				}
			}
		}
	}
	return 1;//not find
}
#endif






