/********************************************************************************************************
 * @file     feature_ll_md.c
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     Nov. 18, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided. 
 *           
 *******************************************************************************************************/
#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"
#include "vendor/common/blt_common.h"

#include "app.h"

#include "app_att.h"
#include "app_device.h"

#include "application/keyboard/keyboard.h"
#include "application/usbstd/usbkeycode.h"


#if (FEATURE_TEST_MODE == TEST_CSA2)


#ifndef LL_RX_OVERFLOW_TEST_ENABLE
#define LL_RX_OVERFLOW_TEST_ENABLE	0
#endif


#if (LL_RX_OVERFLOW_TEST_ENABLE)
static u32 block_tick = 0;
#endif

#define FEATURE_SLAVE_SMP_ENABLE					0
#define FEATURE_MASTER_SMP_ENABLE					0


#define UI_LED_ENABLE			1

#if (UI_LED_ENABLE)
	#define	GPIO_LED_RED			GPIO_PD5
	#define	GPIO_LED_WHITE			GPIO_PD4
	#define	GPIO_LED_GREEN			GPIO_PD3
	#define	GPIO_LED_BLUE			GPIO_PD2

	#define LED_ON_LEVAL 			1 		//gpio output high voltage to turn on led

	const u32 gpio_table[4] = {GPIO_LED_RED, GPIO_LED_WHITE, GPIO_LED_GREEN, GPIO_LED_BLUE};

#endif




MYFIFO_INIT(blt_rxfifo, 64, 16);


/*Note: different connections have their own TX FIFO, master and slave set independently
 * Tx FIFO size = Extra_Len(10) + then align 4bytes
 */
MULTI_CONN_FIFO_INIT(blt_m_txfifo, 40, 8, MASTER_MAX_NUM);  //master connection TX FIFO,  40*8*MASTER_MAX_NUM
MULTI_CONN_FIFO_INIT(blt_s_txfifo, 40, 8, SLAVE_MAX_NUM);   //slave  connection TX FIFO,  40*8*SLAVE_MAX_NUM

#define	MTU_M_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(23)
#define MTU_S_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(23) //if support LE Secure Connections, MTU must more than 65

_attribute_data_retention_	u8 mtu_m_rx_fifo[MASTER_MAX_NUM * MTU_M_BUFF_SIZE_MAX];

_attribute_data_retention_	u8 mtu_s_rx_fifo[SLAVE_MAX_NUM * MTU_S_BUFF_SIZE_MAX];
_attribute_data_retention_	u8 mtu_s_tx_fifo[SLAVE_MAX_NUM * MTU_S_BUFF_SIZE_MAX];


const u8	tbl_advData[] = {
	 0x08, 0x09, 'M', 'S', '-', 'C', 'S', 'A', '2',
	 0x02, 0x01, 0x05, 							// BLE limited discoverable mode and BR/EDR not supported
	 0x03, 0x19, 0x80, 0x01, 					// 384, Generic Remote Control, Generic category
	 0x05, 0x02, 0x12, 0x18, 0x0F, 0x18,		// incomplete list of service class UUIDs (0x1812, 0x180F)
};

const u8	tbl_scanRsp [] = {
	 0x08, 0x09, 'M', 'S', '-', 'C', 'S', 'A', '2',
};


int	master_pairing_enable = 1;


int app_controller_event_callback (u32 h, u8 *p, int n);
int app_host_event_callback (u32 h, u8 *para, int n);


int app_le_adv_report_event_handle(u8 *p)
{
	event_adv_report_t *pa = (event_adv_report_t *)p;
	s8 rssi = pa->data[pa->len];




	int master_auto_connect = 0;
	int user_manual_paring = 0;

	//manual paring methods 1: key press triggers
	user_manual_paring = master_pairing_enable && (rssi > -56);  //button trigger pairing(RSSI threshold, short distance)


	if(master_auto_connect || user_manual_paring){

		blc_llms_createConnection( SCAN_INTERVAL_100MS, SCAN_INTERVAL_100MS, INITIATE_FP_ADV_SPECIFY,  \
								   pa->adr_type, pa->mac, BLE_ADDR_PUBLIC, \
								   CONN_INTERVAL_31P25MS, CONN_INTERVAL_31P25MS, 0, CONN_TIMEOUT_4S, \
								   0, 0xFFFF);
	}

	return 0;
}



int app_le_connection_complete_event_handle(u8 *p)
{
	event_connection_complete_t *pCon = (event_connection_complete_t *)p;

	if(pCon->status == BLE_SUCCESS){

		memset(&cur_conn_device, 0, sizeof(dev_char_info_t));

		//save current connect address type and address and conn_handle
		cur_conn_device.conn_handle = pCon->handle;
		cur_conn_device.peer_adrType = pCon->peer_adr_type;
		memcpy(cur_conn_device.peer_addr, pCon->mac, 6);


		dev_char_info_insert( &cur_conn_device );


#if (UI_LED_ENABLE)
		u8 conn_idx = pCon->handle & CONN_IDX_MASK;
		if(conn_idx < BLMS_MAX_CONN_MASTER_NUM){  //master
			gpio_write( gpio_table[conn_idx], LED_ON_LEVAL);
		}
#endif

	}

	return 0;
}





int 	app_disconnect_event_handle(u8 *p)
{
	event_disconnection_t	*pd = (event_disconnection_t *)p;

	//terminate reason
	if(pd->reason == HCI_ERR_CONN_TIMEOUT){  	//connection timeout

	}
	else if(pd->reason == HCI_ERR_REMOTE_USER_TERM_CONN){  	//peer device send terminate command on link layer

	}
	//master host disconnect( blm_ll_disconnect(current_connHandle, HCI_ERR_REMOTE_USER_TERM_CONN) )
	else if(pd->reason == HCI_ERR_CONN_TERM_BY_LOCAL_HOST){

	}
	else{

	}


	dev_char_info_delete_by_connhandle(pd->connHandle);

#if (UI_LED_ENABLE)

		u8 conn_idx = pd->connHandle & CONN_IDX_MASK;
		if(conn_idx < BLMS_MAX_CONN_MASTER_NUM){  //master
			gpio_write( gpio_table[conn_idx], !LED_ON_LEVAL);
		}
#endif

	return 0;
}



//////////////////////////////////////////////////////////
// event call back
//////////////////////////////////////////////////////////
int app_controller_event_callback (u32 h, u8 *p, int n)
{
	if (h &HCI_FLAG_EVENT_BT_STD)		//Controller HCI event
	{
		u8 evtCode = h & 0xff;

		//------------ disconnect -------------------------------------
		if(evtCode == HCI_EVT_DISCONNECTION_COMPLETE)  //connection terminate
		{
			app_disconnect_event_handle(p);
		}
		else if(evtCode == HCI_EVT_LE_META)  //LE Event
		{
			u8 subEvt_code = p[0];

			//------hci le event: le connection complete event---------------------------------
			if (subEvt_code == HCI_SUB_EVT_LE_CONNECTION_COMPLETE)	// connection complete
			{
				app_le_connection_complete_event_handle(p);

			}
			//------hci le event: le connection establish event---------------------------------
			else if(subEvt_code == HCI_SUB_EVT_LE_CONNECTION_ESTABLISH)
			{
				//	blc_le_connection_establish_event_handle(p);
			}
			//--------hci le event: le adv report event ----------------------------------------
			else if (subEvt_code == HCI_SUB_EVT_LE_ADVERTISING_REPORT)	// ADV packet
			{
				app_le_adv_report_event_handle(p);
			}
			//------hci le event: le connection update complete event-------------------------------
			else if (subEvt_code == HCI_SUB_EVT_LE_CONNECTION_UPDATE_COMPLETE)	// connection update
			{

			}
		}
	}


	return 0;

}





int app_host_event_callback (u32 h, u8 *para, int n)
{
	u8 event = h & 0xFF;

	switch(event)
	{
		case GAP_EVT_SMP_PARING_BEAGIN:
		{

		}
		break;

		case GAP_EVT_SMP_PARING_SUCCESS:
		{

		}
		break;

		case GAP_EVT_SMP_PARING_FAIL:
		{

		}
		break;

		case GAP_EVT_SMP_CONN_ENCRYPTION_DONE:
		{

		}
		break;


		case GAP_EVT_ATT_EXCHANGE_MTU:
		{

		}
		break;



		default:
		break;
	}

	return 0;
}





///////////////////////////////////////////
void feature_csa2_test_init(void)
{
	//random number generator must be initiated here( in the beginning of user_init_nromal)
	//when deepSleep retention wakeUp, no need initialize again
	random_generator_init();  //this is must


///////////// BLE stack Initialization ////////////////
	//for 512K Flash, flash_sector_mac_address equals to 0x76000
	//for 1M   Flash, flash_sector_mac_address equals to 0xFF000
	u8  mac_public[6];
	u8  mac_random_static[6];
	blc_initMacAddress(flash_sector_mac_address, mac_public, mac_random_static);


	////// Controller Initialization  //////////
	blc_llms_initBasicMCU();
	blc_llms_initStandby_module(mac_public);						   //mandatory

	blc_llms_initAdvertising_module(); 	    //adv module: 		 mandatory for BLE slave,

	blc_llms_initScanning_module(); 		//scan module: 		 mandatory for BLE master,

	blc_llms_initMultiMasterMultiSlave_module();


	blc_ll_initChannelSelectionAlgorithm_2_feature();

	blc_llms_setMaxConnectionNumber( MASTER_MAX_NUM, SLAVE_MAX_NUM);



	rf_set_power_level_index (RF_POWER_P3p01dBm);



	////// HCI Initialization  //////////
	blc_hci_registerControllerDataHandler (blt_l2cap_pktHandler);

	blc_hci_registerControllerEventHandler(app_controller_event_callback); //controller hci event to host all processed in this func

	//bluetooth event
	blc_hci_setEventMask_cmd (HCI_EVT_MASK_DISCONNECTION_COMPLETE);

	//bluetooth low energy(LE) event
	blc_hci_le_setEventMask_cmd(		HCI_LE_EVT_MASK_CONNECTION_COMPLETE  \
									|	HCI_LE_EVT_MASK_ADVERTISING_REPORT \
									|   HCI_LE_EVT_MASK_CONNECTION_UPDATE_COMPLETE \
									|   HCI_LE_EVT_MASK_CONNECTION_ESTABLISH);  //connection establish: telink private event   TODO: delete this cause we will support standard HCI

	////// Host Initialization  //////////
	blc_gap_init();    //GAP initialization must be done before any other host feature initialization !!!
	//l2cap initialization
	blm_l2cap_initMtuBuffer(mtu_m_rx_fifo, MTU_M_BUFF_SIZE_MAX, 0,0);
	bls_l2cap_initMtuBuffer(mtu_s_rx_fifo, MTU_S_BUFF_SIZE_MAX, mtu_s_tx_fifo, MTU_S_BUFF_SIZE_MAX);
	my_gatt_init();  //gatt initialization


	//SMP initialization
	#if (FEATURE_SLAVE_SMP_ENABLE)  //Slave SMP Enable
		blc_smp_setSecurityLevel_slave(Unauthenticated_Paring_with_Encryption);  //LE_Security_Mode_1_Level_2
	#else
		blc_smp_setSecurityLevel_slave(No_Security);
	#endif


	#if (FEATURE_MASTER_SMP_ENABLE) //Master SMP Enable
		blc_smp_setSecurityLevel_master(Unauthenticated_Paring_with_Encryption);  //LE_Security_Mode_1_Level_2
	#else
		blc_smp_setSecurityLevel_master(No_Security);
	#endif

	blc_smp_smpParamInit();

	//host(GAP/SMP/GATT/ATT) event process: register host event callback and set event mask
	blc_gap_registerHostEventHandler( app_host_event_callback );
	blc_gap_setEventMask( GAP_EVT_MASK_SMP_PARING_BEAGIN 			|  \
						  GAP_EVT_MASK_SMP_PARING_SUCCESS   		|  \
						  GAP_EVT_MASK_SMP_PARING_FAIL				|  \
						  GAP_EVT_MASK_SMP_SECURITY_PROCESS_DONE);






	////// ADV and Scan CONFIG  //////////
	blc_llms_setAdvData( (u8 *)tbl_advData, sizeof(tbl_advData) );
	blc_llms_setScanRspData( (u8 *)tbl_scanRsp, sizeof(tbl_scanRsp));



	blc_llms_setAdvParam(  ADV_INTERVAL_20MS, ADV_INTERVAL_20MS,
											 ADV_TYPE_CONNECTABLE_UNDIRECTED, OWN_ADDRESS_PUBLIC,
											 0,  NULL,
											 BLT_ENABLE_ADV_ALL,
											 ADV_FP_NONE);
	blc_llms_setAdvEnable(1);  //ADV enable


	blc_llms_setScanParameter(SCAN_TYPE_PASSIVE, SCAN_INTERVAL_200MS, SCAN_WINDOW_200MS, OWN_ADDRESS_PUBLIC, SCAN_FP_ALLOW_ADV_ANY);
	blc_llms_setScanEnable (BLC_SCAN_ENABLE, DUP_FILTER_DISABLE);


	blc_llms_setMaxConnectionNumber(4, 3);



#if (UI_LED_ENABLE)

	gpio_set_func(GPIO_LED_RED,   AS_GPIO);
	gpio_set_func(GPIO_LED_WHITE, AS_GPIO);
	gpio_set_func(GPIO_LED_GREEN,   AS_GPIO);
	gpio_set_func(GPIO_LED_BLUE, AS_GPIO);


	gpio_set_output_en(GPIO_LED_RED, 1);
	gpio_set_output_en(GPIO_LED_WHITE, 1);
	gpio_set_output_en(GPIO_LED_GREEN, 1);
	gpio_set_output_en(GPIO_LED_BLUE, 1);

#endif
}




#endif
