/********************************************************************************************************
 * @file     feature_dle.c
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     Nov. 18, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided. 
 *           
 *******************************************************************************************************/
#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"
#include "vendor/common/blt_common.h"

#include "app.h"

#include "app_att.h"
#include "app_device.h"


#if (FEATURE_TEST_MODE == TEST_LL_DLE)




#define MTU_SIZE_SETTING   			 247
#define DLE_TX_SUPPORTED_DATA_LEN    MAX_OCTETS_DATA_LEN_EXTENSION //264-12 = 252 > Tx max:251


//MYFIFO_INIT(blt_rxfifo, 64, 16);
MYFIFO_INIT(blt_rxfifo, 288, 8);

/*Note: different connections have their own TX FIFO, master and slave set independently
 * Tx FIFO size = Extra_Len(10) + then align 4bytes
 */
MULTI_CONN_FIFO_INIT(blt_m_txfifo, 264, 8, MASTER_MAX_NUM);  //master connection TX FIFO,  40*8*MASTER_MAX_NUM
MULTI_CONN_FIFO_INIT(blt_s_txfifo, 264, 16, SLAVE_MAX_NUM);   //slave  connection TX FIFO,  40*8*SLAVE_MAX_NUM



#define	MTU_M_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(247)
#define MTU_S_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(247) //if support LE Secure Connections, MTU must more than 65

_attribute_data_retention_	u8 mtu_m_rx_fifo[MASTER_MAX_NUM * MTU_M_BUFF_SIZE_MAX];

_attribute_data_retention_	u8 mtu_s_rx_fifo[SLAVE_MAX_NUM * MTU_S_BUFF_SIZE_MAX];
_attribute_data_retention_	u8 mtu_s_tx_fifo[SLAVE_MAX_NUM * MTU_S_BUFF_SIZE_MAX];



#define UI_LED_ENABLE			1

#if (UI_LED_ENABLE)
	#define	GPIO_LED_RED			GPIO_PD5
	#define	GPIO_LED_WHITE			GPIO_PD4
	#define	GPIO_LED_GREEN			GPIO_PD3
	#define	GPIO_LED_BLUE			GPIO_PD2

	#define LED_ON_LEVAL 			1 		//gpio output high voltage to turn on led

	const u32 gpio_table[4] = {GPIO_LED_RED, GPIO_LED_WHITE, GPIO_LED_GREEN, GPIO_LED_BLUE};

#endif





int	master_smp_pending = 0; 		// SMP: security & encryption;
int	master_sdp_pending = 0;			// SDP: service discovery



_attribute_data_retention_	u8	app_conn_flag[BLMS_MAX_CONN_NUM];
_attribute_data_retention_  u32 connect_event_occurTick[BLMS_MAX_CONN_NUM];
_attribute_data_retention_  u32 mtuExchange_check_tick[BLMS_MAX_CONN_NUM];

_attribute_data_retention_ u8 mtuExchange_started_flg[BLMS_MAX_CONN_NUM];
_attribute_data_retention_ u8 dle_started_flg[BLMS_MAX_CONN_NUM];
_attribute_data_retention_ u16 final_MTU_size;

int	master_pairing_enable = 1;


u32 mtu_tick[BLMS_MAX_CONN_NUM];
#define TEST_DATA_LEN		256
_attribute_data_retention_	u8	app_test_data[BLMS_MAX_CONN_NUM][TEST_DATA_LEN];


const u8	tbl_advData[] = {
	 0x09, 0x09, 'D', 'L', 'E', '_', 'T', 'e', 's', 't',     //device name set longer on purpose, use maximum 31 byte PDU to test ADV timing at debug stage
	 0x02, 0x01, 0x05, 							// BLE limited discoverable mode and BR/EDR not supported
	 0x03, 0x19, 0x80, 0x01, 					// 384, Generic Remote Control, Generic category
	 0x05, 0x02, 0x12, 0x18, 0x0F, 0x18,		// incomplete list of service class UUIDs (0x1812, 0x180F)
};

const u8	tbl_scanRsp [] = {
	 0x09, 0x09, 'D', 'L', 'E', '_', 'T', 'e', 's', 't',    //device name set longer on purpose, use maximum 31 byte PDU to test ADV timing at debug stage
	 0x02, 0x01, 0x05, 							// BLE limited discoverable mode and BR/EDR not supported
	 0x03, 0x19, 0x80, 0x01, 					// 384, Generic Remote Control, Generic category
	 0x05, 0x02, 0x12, 0x18, 0x0F, 0x18,		// incomplete list of service class UUIDs (0x1812, 0x180F)
};




const u8 	telink_adv_trigger_paring[] = {5, 0xFF, 0x11, 0x02, 0x01, 0x00};
const u8 	telink_adv_trigger_unpair[] = {5, 0xFF, 0x11, 0x02, 0x01, 0x01};

const u8 	telink_adv_trigger_paring_8258[] = {7, 0xFF, 0x11, 0x02, 0x01, 0x00, 0x58, 0x82};
const u8 	telink_adv_trigger_unpair_8258[] = {7, 0xFF, 0x11, 0x02, 0x01, 0x01, 0x58, 0x82};



int app_controller_event_callback (u32 h, u8 *p, int n);
int app_host_event_callback (u32 h, u8 *para, int n);
int app_gatt_data_handler (u16 connHandle, u8 *pkt);






int app_le_adv_report_event_handle(u8 *p)
{

	event_adv_report_t *pa = (event_adv_report_t *)p;
	s8 rssi = pa->data[pa->len];


	int master_auto_connect = 0;
	int user_manual_paring = 0;

	//manual paring methods 1: key press triggers
	user_manual_paring = master_pairing_enable && (rssi > -56);  //button trigger pairing(RSSI threshold, short distance)


	if(master_auto_connect || user_manual_paring){

		blc_llms_createConnection( SCAN_INTERVAL_100MS, SCAN_INTERVAL_100MS, INITIATE_FP_ADV_SPECIFY,  \
								   pa->adr_type, pa->mac, BLE_ADDR_PUBLIC, \
								   CONN_INTERVAL_31P25MS, CONN_INTERVAL_31P25MS, 0, CONN_TIMEOUT_4S, \
								   0, 0xFFFF);
	}

	return 0;
}



int app_le_connection_complete_event_handle(u8 *p)
{
	event_connection_complete_t *pCon = (event_connection_complete_t *)p;

	if(pCon->status == BLE_SUCCESS){

		memset(&cur_conn_device, 0, sizeof(dev_char_info_t));

		//save current connect address type and address and conn_handle
		cur_conn_device.conn_handle = pCon->handle;
		cur_conn_device.peer_adrType = pCon->peer_adr_type;
		memcpy(cur_conn_device.peer_addr, pCon->mac, 6);


		dev_char_info_insert( &cur_conn_device );

		u8 conn_idx = pCon->handle & CONN_IDX_MASK;
		app_conn_flag[conn_idx]    = 1;

		connect_event_occurTick[conn_idx] = clock_time()|0x01;
		if( pCon->handle & BLM_CONN_HANDLE ) // master connection handle, process SMP and SDP
		{
			#if (BLE_MASTER_SIMPLE_SDP_ENABLE)
				u8	temp_buff[sizeof(dev_att_t)];
				dev_att_t *pdev_att = (dev_att_t *)temp_buff;
				if( dev_char_info_search_peer_att_handle_by_peer_mac(pCon->peer_adr_type, pCon->mac, pdev_att) ){  //att_handle search in flash success, load char_handle directly from flash, no need SDP again
					cur_conn_device.char_handle[2] = pdev_att->char_handle[2];			//OTA
					cur_conn_device.char_handle[3] = pdev_att->char_handle[3];			//consume report
					cur_conn_device.char_handle[4] = pdev_att->char_handle[4];			//normal key report
					dev_char_info_add_peer_att_handle(&cur_conn_device);  //add the peer device att_handle value to conn_dev_list
				}
				else{
					master_sdp_pending = pCon->handle;  // mark this connection need SDP
					#if (BLE_MASTER_SMP_ENABLE)
					#else
						 app_register_service(&app_service_discovery);
					#endif
				}
			#endif
		}

#if (UI_LED_ENABLE)
		if(conn_idx < BLMS_MAX_CONN_MASTER_NUM){  //master
			gpio_write( gpio_table[conn_idx], LED_ON_LEVAL);
		}
#endif
	}

	return 0;
}





int 	app_disconnect_event_handle(u8 *p)
{
	event_disconnection_t	*pd = (event_disconnection_t *)p;

	//terminate reason
	if(pd->reason == HCI_ERR_CONN_TIMEOUT){  	//connection timeout

	}
	else if(pd->reason == HCI_ERR_REMOTE_USER_TERM_CONN){  	//peer device send terminate command on link layer

	}
	//master host disconnect( blm_ll_disconnect(current_connHandle, HCI_ERR_REMOTE_USER_TERM_CONN) )
	else if(pd->reason == HCI_ERR_CONN_TERM_BY_LOCAL_HOST){

	}
	else{

	}


	dev_char_info_delete_by_connhandle(pd->connHandle);

	u8 conn_idx = pd->connHandle & CONN_IDX_MASK;
	app_conn_flag[conn_idx]    = 0;

	connect_event_occurTick[conn_idx] = 0;

	mtuExchange_check_tick[conn_idx] = 0;

	dle_started_flg[conn_idx] = 0;

	mtuExchange_started_flg[conn_idx] = 0;

#if (UI_LED_ENABLE)
		if(conn_idx < BLMS_MAX_CONN_MASTER_NUM){  //master
			gpio_write( gpio_table[conn_idx], !LED_ON_LEVAL);
		}
#endif


	return 0;
}



//////////////////////////////////////////////////////////
// event call back
//////////////////////////////////////////////////////////
int app_controller_event_callback (u32 h, u8 *p, int n)
{
	if (h &HCI_FLAG_EVENT_BT_STD)		//Controller HCI event
	{
		u8 evtCode = h & 0xff;

		//------------ disconnect -------------------------------------
		if(evtCode == HCI_EVT_DISCONNECTION_COMPLETE)  //connection terminate
		{
			app_disconnect_event_handle(p);
		}
		else if(evtCode == HCI_EVT_LE_META)  //LE Event
		{
			u8 subEvt_code = p[0];

			//------hci le event: le connection complete event---------------------------------
			if (subEvt_code == HCI_SUB_EVT_LE_CONNECTION_COMPLETE)	// connection complete
			{
				//after controller is set to initiating state by host (blc_llms_createConnection(...) )
				//it will scan the specified device(adr_type & mac), when find this adv packet, send a connection request packet to slave
				//and enter to connection state, send connection complete event. but notice that connection complete not
				//equals to connection establish. connection complete means that master controller set all the ble timing
				//get ready, but has not received any slave packet, if slave rf lost the connection request packet, it will
				//not send any packet to master controller

				app_le_connection_complete_event_handle(p);

			}
			//------hci le event: le connection establish event---------------------------------
			else if(subEvt_code == HCI_SUB_EVT_LE_CONNECTION_ESTABLISH)  //connection establish(telink private event)
			{
				//notice that: this connection event is defined by telink, not a standard ble controller event
				//after master controller send connection request packet to slave, when slave received this packet
				//and enter to connection state, send a ack packet within 6 connection event, master will send
				//connection establish event to host(HCI_SUB_EVT_LE_CONNECTION_ESTABLISH)

				//	blc_le_connection_establish_event_handle(p);
			}
			//--------hci le event: le adv report event ----------------------------------------
			else if (subEvt_code == HCI_SUB_EVT_LE_ADVERTISING_REPORT)	// ADV packet
			{
				//after controller is set to scan state, it will report all the adv packet it received by this event

				app_le_adv_report_event_handle(p);
			}
			//------hci le event: le connection update complete event-------------------------------
			else if (subEvt_code == HCI_SUB_EVT_LE_CONNECTION_UPDATE_COMPLETE)	// connection update
			{

			}
			else if(subEvt_code == HCI_SUB_EVT_LE_DATA_LENGTH_CHANGE){
				hci_le_dataLengthChangeEvt_t* dle_param = (hci_le_dataLengthChangeEvt_t*)p;
				u8 conn_idx = dle_param->connHandle & CONN_IDX_MASK;

				dle_started_flg[conn_idx] = 1;
			}
		}
	}
	return 0;
}





int app_host_event_callback (u32 h, u8 *para, int n)
{
	u8 event = h & 0xFF;

	switch(event)
	{
		case GAP_EVT_SMP_PARING_BEAGIN:
		{

		}
		break;

		case GAP_EVT_SMP_PARING_SUCCESS:
		{

		}
		break;

		case GAP_EVT_SMP_PARING_FAIL:
		{

		}
		break;

		case GAP_EVT_SMP_CONN_ENCRYPTION_DONE:
		{

		}
		break;


		case GAP_EVT_ATT_EXCHANGE_MTU:
		{
			gap_gatt_mtuSizeExchangeEvt_t *pEvt = (gap_gatt_mtuSizeExchangeEvt_t *)para; ///gap_gatt_mtuSizeExchangeEvt_t
			u8 conn_idx = pEvt->connHandle & CONN_IDX_MASK;
			final_MTU_size = pEvt->effective_MTU;
			mtuExchange_started_flg[conn_idx] = 1;   //set MTU size exchange flag here

//			printf("GAP_EVT_ATT_EXCHANGE_MTU, h: %2x, mtu: %2x, peer: %2x", pEvt->connHandle, pEvt->effective_MTU, pEvt->peer_MTU);
		}
		break;



		default:
		break;
	}

	return 0;
}





int app_gatt_data_handler (u16 connHandle, u8 *pkt)
{

	if( connHandle & BLM_CONN_HANDLE)   //GATT data for Master
	{
			#if (BLE_MASTER_SIMPLE_SDP_ENABLE)
				if(master_sdp_pending == connHandle ){  //ATT service discovery is ongoing on this conn_handle
					//when service discovery function is running, all the ATT data from slave
					//will be processed by it,  user can only send your own att cmd after  service discovery is over
					host_att_client_handler (connHandle, pkt); //handle this ATT data by service discovery process
				}
			#endif

			//so any ATT data before service discovery will be dropped
			dev_char_info_t* dev_info = dev_char_info_search_by_connhandle (connHandle);
			if(dev_info)
			{
				//-------	user process ------------------------------------------------
				rf_packet_att_t *pAtt = (rf_packet_att_t*)pkt;
				u16 attHandle = pAtt->handle0 | pAtt->handle1<<8;

				if(pAtt->opcode == ATT_OP_HANDLE_VALUE_NOTI)  //slave handle notify
				{
						//---------------	consumer key --------------------------
					#if (BLE_MASTER_SIMPLE_SDP_ENABLE)
						if(attHandle == dev_info->char_handle[3])  // Consume Report In (Media Key)
					#else
						if(attHandle == 25)   //Demo device(825x_ble_sample) Consume Report AttHandle value is 25
					#endif
						{
//							att_keyboard_media (connHandle, pAtt->dat);
						}
						//---------------	keyboard key --------------------------
					#if (BLE_MASTER_SIMPLE_SDP_ENABLE)
						else if(attHandle == dev_info->char_handle[4])     // Key Report In
					#else
						else if(0)   // Demo device(825x_ble_sample) Key Report AttHandle value is 29
					#endif
						{
							att_keyboard (connHandle, pAtt->dat);
						}
						else
						{

						}
				}
				else if (pAtt->opcode == ATT_OP_HANDLE_VALUE_IND)
				{

				}
			}

	}
	else{   //GATT data for Slave


	}


	return 0;
}







///////////////////////////////////////////
void feature_dle_test_init(void)
{
	//random number generator must be initiated here( in the beginning of user_init_nromal)
	//when deepSleep retention wakeUp, no need initialize again
	random_generator_init();  //this is must


///////////// BLE stack Initialization ////////////////
	//for 512K Flash, flash_sector_mac_address equals to 0x76000
	//for 1M   Flash, flash_sector_mac_address equals to 0xFF000
	u8  mac_public[6];
	u8  mac_random_static[6];
	blc_initMacAddress(flash_sector_mac_address, mac_public, mac_random_static);


	////// Controller Initialization  //////////
	blc_llms_initBasicMCU();
	blc_llms_initStandby_module(mac_public);						   //mandatory

	blc_llms_initAdvertising_module(); 	    //adv module: 		 mandatory for BLE slave,

	blc_llms_initScanning_module(); 		//scan module: 		 mandatory for BLE master,

	blc_llms_initMultiMasterMultiSlave_module();


	blc_llms_setMaxConnectionNumber( MASTER_MAX_NUM, SLAVE_MAX_NUM);



	rf_set_power_level_index (RF_POWER_P3p01dBm);

	////// HCI Initialization  //////////
	blc_hci_registerControllerDataHandler (blt_l2cap_pktHandler);

	blc_hci_registerControllerEventHandler(app_controller_event_callback); //controller hci event to host all processed in this func

	//bluetooth event
	blc_hci_setEventMask_cmd (HCI_EVT_MASK_DISCONNECTION_COMPLETE);

	//bluetooth low energy(LE) event
	blc_hci_le_setEventMask_cmd(		HCI_LE_EVT_MASK_CONNECTION_COMPLETE  \
									|	HCI_LE_EVT_MASK_ADVERTISING_REPORT \
									|   HCI_LE_EVT_MASK_CONNECTION_UPDATE_COMPLETE \
									|   HCI_LE_EVT_MASK_DATA_LENGTH_CHANGE \
									|   HCI_LE_EVT_MASK_CONNECTION_ESTABLISH );  //connection establish: telink private event   TODO: delete this cause we will support standard HCI







	////// Host Initialization  //////////
	blc_gap_init();    //GAP initialization must be done before any other host feature initialization !!!
	//l2cap initialization
	blm_l2cap_initMtuBuffer(mtu_m_rx_fifo, MTU_M_BUFF_SIZE_MAX, 0,0);
	bls_l2cap_initMtuBuffer(mtu_s_rx_fifo, MTU_S_BUFF_SIZE_MAX, mtu_s_tx_fifo, MTU_S_BUFF_SIZE_MAX);
	my_gatt_init();  //gatt initialization

	blm_l2cap_setRxMTUSize(MTU_SIZE_SETTING);
	bls_l2cap_setRxMTUSize(MTU_SIZE_SETTING);

	#if (BLE_MASTER_SIMPLE_SDP_ENABLE)
		host_att_register_idle_func (main_idle_loop);
	#endif


	blc_gatt_register_data_handler(app_gatt_data_handler);


	//SMP initialization
	#if (FEATURE_SLAVE_SMP_ENABLE)  //Slave SMP Enable
		blc_smp_setSecurityLevel_slave(Unauthenticated_Paring_with_Encryption);  //LE_Security_Mode_1_Level_2
	#else
		blc_smp_setSecurityLevel_slave(No_Security);
	#endif


	#if (FEATURE_MASTER_SMP_ENABLE) //Master SMP Enable
		blc_smp_setSecurityLevel_master(Unauthenticated_Paring_with_Encryption);  //LE_Security_Mode_1_Level_2
	#else
		blc_smp_setSecurityLevel_master(No_Security);
	#endif

	blc_smp_smpParamInit();

	//host(GAP/SMP/GATT/ATT) event process: register host event callback and set event mask
	blc_gap_registerHostEventHandler( app_host_event_callback );
	blc_gap_setEventMask( GAP_EVT_MASK_SMP_PARING_BEAGIN 			|  \
						  GAP_EVT_MASK_SMP_PARING_SUCCESS   		|  \
						  GAP_EVT_MASK_SMP_PARING_FAIL				|  \
						  GAP_EVT_MASK_SMP_SECURITY_PROCESS_DONE	|  \
						  GAP_EVT_MASK_ATT_EXCHANGE_MTU);

	blc_llms_setAdvData( (u8 *)tbl_advData, sizeof(tbl_advData) );
	blc_llms_setScanRspData( (u8 *)tbl_scanRsp, sizeof(tbl_scanRsp));
	// ADV_INTERVAL_25MS
	blc_llms_setAdvParam(  ADV_INTERVAL_30MS, ADV_INTERVAL_30MS,
											 ADV_TYPE_CONNECTABLE_UNDIRECTED, OWN_ADDRESS_PUBLIC,
											 0,  NULL,
											 BLT_ENABLE_ADV_ALL,
											 ADV_FP_NONE);
	blc_llms_setAdvEnable(1);  //ADV enable


	blc_llms_setScanParameter(SCAN_TYPE_PASSIVE, SCAN_INTERVAL_200MS, SCAN_WINDOW_200MS, OWN_ADDRESS_PUBLIC, SCAN_FP_ALLOW_ADV_ANY);
	blc_llms_setScanEnable (BLC_SCAN_ENABLE, DUP_FILTER_DISABLE);


#if (UI_LED_ENABLE)

	gpio_set_func(GPIO_LED_RED,   AS_GPIO);
	gpio_set_func(GPIO_LED_WHITE, AS_GPIO);
	gpio_set_func(GPIO_LED_GREEN,   AS_GPIO);
	gpio_set_func(GPIO_LED_BLUE, AS_GPIO);


	gpio_set_output_en(GPIO_LED_RED, 1);
	gpio_set_output_en(GPIO_LED_WHITE, 1);
	gpio_set_output_en(GPIO_LED_GREEN, 1);
	gpio_set_output_en(GPIO_LED_BLUE, 1);

#endif
}



/////////////////////////////////////////////////////////////////////
// main loop flow
/////////////////////////////////////////////////////////////////////
void feature_dle_test_mainloop (void)
{
	u16 connHandle;
	for(u8 i= 0; i < BLMS_MAX_CONN_NUM; i++){

		if(app_conn_flag[i]){
			if(i < BLMS_MAX_CONN_MASTER_NUM){
				connHandle = BLM_CONN_HANDLE | i;
			}
			else{
				connHandle = BLS_CONN_HANDLE | i;
			}
#if 1
			///send LL_LENGTH_REQ to peer device. need to exchange MTU, too.
			if(connect_event_occurTick[i] && clock_time_exceed(connect_event_occurTick[i], 1500000)){  //1.5 S after connection established
				connect_event_occurTick[i] = 0;

				mtuExchange_check_tick[i] = clock_time() | 1;

				if(!mtuExchange_started_flg[i]){
//					printf("send MTU");
					blc_att_requestMtuSizeExchange(connHandle, MTU_SIZE_SETTING); ///blms_rxbuff
				}
			}

			/////DLE exchange
			if(mtuExchange_check_tick[i] && clock_time_exceed(mtuExchange_check_tick[i], 500000 )){  //2 S after connection established

				mtuExchange_check_tick[i] = 0;
				if(!dle_started_flg[i]){
					blc_llms_exchangeDataLength(connHandle, LL_LENGTH_REQ , DLE_TX_SUPPORTED_DATA_LEN );
				}
			}


			//send data to peer device
			if(dle_started_flg[i] && clock_time_exceed(mtu_tick[i], 100000))
			{
				if(i < BLMS_MAX_CONN_MASTER_NUM)
				{
					if(BLE_SUCCESS ==blc_gatt_pushWriteCommand (connHandle, SPP_CLIENT_TO_SERVER_DP_H, &app_test_data[i][0], MTU_SIZE_SETTING-3))
					{
						mtu_tick[i] = clock_time();
						app_test_data[i][0] ++;
					}
				}
				else
				{
					if(BLE_SUCCESS == blc_gatt_pushHandleValueNotify (connHandle, SPP_SERVER_TO_CLIENT_DP_H, &app_test_data[i][0], MTU_SIZE_SETTING-3))
					{
						mtu_tick[i] = clock_time();
						app_test_data[i][0] ++;
					}
				}
			}
#endif

		}

	}

}







#endif
