/********************************************************************************************************
 * @file     feature_gatt_api_test.c
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     May. 22, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/
#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"
#include "vendor/common/blt_common.h"

#include "app_att.h"
#include "app_device.h"

#if(FEATURE_TEST_MODE == TEST_GATT_API)

//all connections share same RX FIFO
MYFIFO_INIT(blt_rxfifo, 64, 16);


/*
 * Note: different connections have their own TX FIFO, master and slave set independently
 * Tx FIFO size = Extra_Len(10) + then align 4bytes
 */
MULTI_CONN_FIFO_INIT(blt_m_txfifo, 40, 8, MASTER_MAX_NUM);  //master connection TX FIFO,  40*8*MASTER_MAX_NUM
MULTI_CONN_FIFO_INIT(blt_s_txfifo, 40, 8, SLAVE_MAX_NUM);   //slave  connection TX FIFO,  40*8*SLAVE_MAX_NUM

#define	MTU_M_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(23)
#define MTU_S_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(23) //if support LE Secure Connections, MTU must more than 65

_attribute_data_retention_	u8 mtu_m_rx_fifo[MASTER_MAX_NUM * MTU_M_BUFF_SIZE_MAX];

_attribute_data_retention_	u8 mtu_s_rx_fifo[SLAVE_MAX_NUM * MTU_S_BUFF_SIZE_MAX];
_attribute_data_retention_	u8 mtu_s_tx_fifo[SLAVE_MAX_NUM * MTU_S_BUFF_SIZE_MAX];

int	master_smp_pending = 0; 		// SMP: security & encryption;
int	master_sdp_pending = 0;			// SDP: service discovery

const u8	tbl_advData[] = {
	 0x09, 0x09, 'T','L','K','_','M','4','S','3',
	 0x02, 0x01, 0x05, 							// BLE limited discoverable mode and BR/EDR not supported
	 0x03, 0x19, 0x80, 0x01, 					// 384, Generic Remote Control, Generic category
	 0x05, 0x02, 0x12, 0x18, 0x0F, 0x18,		// incomplete list of service class UUIDs (0x1812, 0x180F)
};

const u8	tbl_scanRsp [] = {
	 0x09, 0x09, 'T','L','K','_','M','4','S','3',
};

int master_pairing_enable = 1;
u16 connHandleTbl[MASTER_MAX_NUM] = {0};

int app_controller_event_callback (u32 h, u8 *p, int n);
int app_host_event_callback (u32 h, u8 *para, int n);
int app_gatt_data_handler (u16 connHandle, u8 *pkt);
int main_idle_loop (void);

int app_le_adv_report_event_handle(u8 *p)
{
	event_adv_report_t *pa = (event_adv_report_t *)p;
	s8 rssi = pa->data[pa->len];

	//// Here user can use ADV report data to do a lot of work ////
	int master_auto_connect = 0;
	int user_manual_paring = 0;

	//manual paring methods 1: key press triggers
	user_manual_paring = master_pairing_enable && (rssi > -56);  //button trigger pairing(RSSI threshold, short distance)

	#if 0
		u8 addr[4] = {0xbb, 0xbb,0xbb,0xbb,};
		if(memcmp(addr, pa->mac+2, 3))
			return 0;
	#endif

	if(master_auto_connect || user_manual_paring){
		blc_llms_createConnection( SCAN_INTERVAL_100MS, SCAN_WINDOW_100MS, INITIATE_FP_ADV_SPECIFY,
								   pa->adr_type, pa->mac, BLE_ADDR_PUBLIC,
								   CONN_INTERVAL_31P25MS, CONN_INTERVAL_31P25MS, 0, CONN_TIMEOUT_4S,
								   0, 0xFFFF);
	}
	return 0;
}


int app_le_connection_complete_event_handle(u8 *p)
{
	event_connection_complete_t *pCon = (event_connection_complete_t *)p;

	if(pCon->status == BLE_SUCCESS){

		dev_char_info_t cur_conn_device;
		memset(&cur_conn_device, 0, sizeof(dev_char_info_t));

		//save current connect address type and address and conn_handle
		cur_conn_device.conn_handle = pCon->handle;
		cur_conn_device.peer_adrType = pCon->peer_adr_type;
		memcpy(cur_conn_device.peer_addr, pCon->mac, 6);

		dev_char_info_insert( &cur_conn_device );

		if( pCon->handle & BLM_CONN_HANDLE ) // master connection handle, process SMP and SDP
		{
			connHandleTbl[pCon->handle & CONN_IDX_MASK] = pCon->handle;
		}
	}

	return 0;
}


int app_disconnect_event_handle(u8 *p)
{
	event_disconnection_t	*pd = (event_disconnection_t *)p;

	//terminate reason
	if(pd->reason == HCI_ERR_CONN_TIMEOUT){  	//connection timeout

	}
	else if(pd->reason == HCI_ERR_REMOTE_USER_TERM_CONN){  	//peer device send terminate command on link layer

	}
	//master host disconnect( blm_ll_disconnect(current_connHandle, HCI_ERR_REMOTE_USER_TERM_CONN) )
	else if(pd->reason == HCI_ERR_CONN_TERM_BY_LOCAL_HOST){

	}
	else{

	}

	//if previous connection SMP & SDP not finished, clear flag
	if(master_smp_pending == pd->connHandle){
		master_smp_pending = 0;
	}
	if(master_sdp_pending == pd->connHandle){
		master_sdp_pending = 0;
	}

	dev_char_info_delete_by_connhandle(pd->connHandle);

	if(pd->connHandle & BLM_CONN_HANDLE){
		connHandleTbl[pd->connHandle & CONN_IDX_MASK] = 0;
	}
	return 0;
}



//////////////////////////////////////////////////////////
// event call back
//////////////////////////////////////////////////////////
int app_controller_event_callback (u32 h, u8 *p, int n)
{
	if (h &HCI_FLAG_EVENT_BT_STD)		//Controller HCI event
	{
		u8 evtCode = h & 0xff;

		//------------ disconnect -------------------------------------
		if(evtCode == HCI_EVT_DISCONNECTION_COMPLETE)  //connection terminate
		{
			app_disconnect_event_handle(p);
		}
		else if(evtCode == HCI_EVT_LE_META)  //LE Event
		{
			u8 subEvt_code = p[0];

			//------hci le event: le connection complete event---------------------------------
			if (subEvt_code == HCI_SUB_EVT_LE_CONNECTION_COMPLETE)	// connection complete
			{
				//after controller is set to initiating state by host (blc_llms_createConnection(...) )
				//it will scan the specified device(adr_type & mac), when find this adv packet, send a connection request packet to slave
				//and enter to connection state, send connection complete event. but notice that connection complete not
				//equals to connection establish. connection complete means that master controller set all the ble timing
				//get ready, but has not received any slave packet, if slave rf lost the connection request packet, it will
				//not send any packet to master controller

				app_le_connection_complete_event_handle(p);
			}
			//------hci le event: le connection establish event---------------------------------
			else if(subEvt_code == HCI_SUB_EVT_LE_CONNECTION_ESTABLISH)  //connection establish(telink private event)
			{
				//notice that: this connection event is defined by telink, not a standard ble controller event
				//after master controller send connection request packet to slave, when slave received this packet
				//and enter to connection state, send a ack packet within 6 connection event, master will send
				//connection establish event to host(HCI_SUB_EVT_LE_CONNECTION_ESTABLISH)

				//	blc_le_connection_establish_event_handle(p);
			}
			//--------hci le event: le adv report event ----------------------------------------
			else if (subEvt_code == HCI_SUB_EVT_LE_ADVERTISING_REPORT)	// ADV packet
			{
				//after controller is set to scan state, it will report all the adv packet it received by this event

				app_le_adv_report_event_handle(p);
			}
			//------hci le event: le connection update complete event-------------------------------
			else if (subEvt_code == HCI_SUB_EVT_LE_CONNECTION_UPDATE_COMPLETE)	// connection update
			{

			}
		}
	}

	return 0;
}

int app_host_event_callback (u32 h, u8 *para, int n)
{
	u8 event = h & 0xFF;

	switch(event)
	{
		case GAP_EVT_SMP_PARING_BEAGIN:
		{
			gap_smp_paringBeginEvt_t *p = (gap_smp_paringBeginEvt_t *)para;
			/*
			 * 	JustWorks = 0, PK_Init_Dsply_Resp_Input = 1, PK_Resp_Dsply_Init_Input = 2, PK_BOTH_INPUT = 3,OOB_Authentication = 4, Numric_Comparison = 5,
			 */
			u8 TK_method[6][40] = {"JustWorks", "PK_Init_Dsply_Resp_Input", "PK_Resp_Dsply_Init_Input", "PK_BOTH_INPUT", "OOB_Authentication", "Numric_Comparison"};
			printf("[0x%x]Pairing begin (sc:%s,TK-method:%s)\n", p->connHandle, p->secure_conn?"SC":"Legacy", TK_method[p->tk_method]);
			printf("[0x%x]Pairing begin\n", p->connHandle);
		}
		break;
		case GAP_EVT_SMP_PARING_SUCCESS:
		{
			gap_smp_paringSuccessEvt_t* p = (gap_smp_paringSuccessEvt_t*)para;
		    printf("[0x%x]Pairing success: bond%s (save %s)\n", p->connHandle, p->bonding ?"T":"F", p->bonding_result ? "OK" : "ERR");
		}
		break;
		case GAP_EVT_SMP_PARING_FAIL:
		{
			gap_smp_paringFailEvt_t* p = (gap_smp_paringFailEvt_t*)para;
			printf("[0x%x]Pairing failed:rsn:0x%x\n", p->connHandle, p->reason);
		}
		break;
		case GAP_EVT_SMP_CONN_ENCRYPTION_DONE:
		{

		}
		break;
		case GAP_EVT_SMP_SECURITY_PROCESS_DONE:
		{
			gap_smp_connEncDoneEvt_t* p = (gap_smp_connEncDoneEvt_t*)para;
			printf("[0x%x]ENC done (%s)\n", p->connHandle, p->re_connect == SMP_STANDARD_PAIR ? "1stParing" :"reConnect");

			if( p->connHandle & BLM_CONN_HANDLE){  //master connection
				#if (BLE_MASTER_SMP_ENABLE)
					if(master_smp_pending == p->connHandle){
						master_smp_pending = 0;
					}
				#endif

				#if (BLE_MASTER_SIMPLE_SDP_ENABLE)  //SMP finish
					if(master_sdp_pending == p->connHandle){  //SDP is pending
						app_register_service(&app_service_discovery);  //start SDP now
					}
				#endif
			}

		}
		break;
		case GAP_EVT_SMP_TK_DISPALY:
		{
			char pc[7];
			gap_smp_TkDisplayEvt_t* p = (gap_smp_TkDisplayEvt_t*)para;
			sprintf(pc, "%d", p->tk_pincode);
			//printf("[0x%x]TK display:%s\n", p->connHandle, pc);
		}
		break;
		case GAP_EVT_SMP_TK_REQUEST_PASSKEY:
		{
			gap_smp_TkReqPassKeyEvt_t* p = (gap_smp_TkReqPassKeyEvt_t*)para;
			//printf("[0x%x]TK Request passkey\n", p->connHandle);
		}
		break;
		case GAP_EVT_SMP_TK_REQUEST_OOB:
		{

		}
		break;
		case GAP_EVT_SMP_TK_NUMERIC_COMPARE:
		{

		}
		break;
		case GAP_EVT_ATT_EXCHANGE_MTU:
		{

		}
		break;
		case GAP_EVT_GATT_HANDLE_VLAUE_CONFIRM:
		{

		}
		break;
		default:
		break;
	}

	return 0;
}

#define			HID_HANDLE_CONSUME_REPORT			25
#define			HID_HANDLE_KEYBOARD_REPORT			29
#define			AUDIO_HANDLE_MIC					52
#define			OTA_HANDLE_DATA						48


/**
 * User can check data by Debug tool of Telink
 */
u8 A_attRspPkt[200];

int app_gatt_data_handler (u16 connHandle, u8 *pkt)
{
	if( connHandle & BLM_CONN_HANDLE)   //GATT data for Master
	{
	#if (BLE_MASTER_SIMPLE_SDP_ENABLE)
		if(master_sdp_pending == connHandle ){  //ATT service discovery is ongoing on this conn_handle
			//when service discovery function is running, all the ATT data from slave
			//will be processed by it,  user can only send your own att cmd after  service discovery is over
			host_att_client_handler (connHandle, pkt); //handle this ATT data by service discovery process
		}
	#endif

		rf_packet_att_t *pAtt = (rf_packet_att_t*)pkt;
		switch(pAtt->opcode)
		{
		case ATT_OP_FIND_INFO_RSP:
			memcpy(A_attRspPkt, (u8*)&pAtt->opcode, min(sizeof(A_attRspPkt), pAtt->l2capLen));
			break;
		case ATT_OP_FIND_BY_TYPE_VALUE_RSP:
			memcpy(A_attRspPkt, (u8*)&pAtt->opcode, min(sizeof(A_attRspPkt), pAtt->l2capLen));
			break;
		case ATT_OP_READ_RSP:
			memcpy(A_attRspPkt, (u8*)&pAtt->opcode, min(sizeof(A_attRspPkt), pAtt->l2capLen));
			break;
		case ATT_OP_READ_BLOB_RSP:
			memcpy(A_attRspPkt, (u8*)&pAtt->opcode, min(sizeof(A_attRspPkt), pAtt->l2capLen));
			break;
		case ATT_OP_READ_BY_TYPE_RSP:
			memcpy(A_attRspPkt, (u8*)&pAtt->opcode, min(sizeof(A_attRspPkt), pAtt->l2capLen));
			break;
		case ATT_OP_READ_BY_GROUP_TYPE_RSP:
			memcpy(A_attRspPkt, (u8*)&pAtt->opcode, min(sizeof(A_attRspPkt), pAtt->l2capLen));
			break;
		default:
			break;
		}
	}
	else{//GATT data for Slave
	}

	return 0;
}

///////////////////////////////////////////
void feature_gatt_api_test_init(void)
{
	//random number generator must be initiated here( in the beginning of user_init_nromal)
	//when deepSleep retention wakeUp, no need initialize again
	random_generator_init();  //this is must

	#if (APPLICATION_DONGLE)
		//set USB ID
		REG_ADDR8(0x74) = 0x62;
		REG_ADDR16(0x7e) = 0x08d0;
		REG_ADDR8(0x74) = 0x00;

		//////////////// config USB ISO IN/OUT interrupt /////////////////
		#if(UI_AUDIO_ENABLE)
			reg_usb_mask = BIT(7);			//audio in interrupt enable
			reg_irq_mask |= FLD_IRQ_IRQ4_EN;
		#endif
		reg_usb_ep6_buf_addr = 0x80;
		reg_usb_ep7_buf_addr = 0x60;
		reg_usb_ep_max_size = (256 >> 3);

		usb_dp_pullup_en (1);  //open USB enum
	#endif

///////////// BLE stack Initialization ////////////////
	u8  mac_public[6];
	u8  mac_random_static[6];
	blc_initMacAddress(flash_sector_mac_address, mac_public, mac_random_static);


	////// Controller Initialization  //////////
	blc_llms_initBasicMCU();
	blc_llms_initStandby_module(mac_public);						   //mandatory

	blc_llms_initAdvertising_module(); 	//adv module: 		 mandatory for BLE slave,

	blc_llms_initScanning_module(); 		//scan module: 		 mandatory for BLE master,

	blc_llms_initMultiMasterMultiSlave_module();

	blc_llms_setMaxConnectionNumber( MASTER_MAX_NUM, SLAVE_MAX_NUM);

	rf_set_power_level_index (RF_POWER_P3p01dBm);

	////// HCI Initialization  //////////
	blc_hci_registerControllerDataHandler (blt_l2cap_pktHandler);

	blc_hci_registerControllerEventHandler(app_controller_event_callback); //controller hci event to host all processed in this func

	//bluetooth event
	blc_hci_setEventMask_cmd (HCI_EVT_MASK_DISCONNECTION_COMPLETE);

	//bluetooth low energy(LE) event
	blc_hci_le_setEventMask_cmd(		HCI_LE_EVT_MASK_CONNECTION_COMPLETE  \
									|	HCI_LE_EVT_MASK_ADVERTISING_REPORT \
									|   HCI_LE_EVT_MASK_CONNECTION_UPDATE_COMPLETE \
									|   HCI_LE_EVT_MASK_CONNECTION_ESTABLISH);  //connection establish: telink private event   TODO: delete this cause we will support standard HCI

	////// Host Initialization  //////////
	blc_gap_init();    //GAP initialization must be done before any other host feature initialization !!!
	//l2cap initialization
	blm_l2cap_initMtuBuffer(mtu_m_rx_fifo, MTU_M_BUFF_SIZE_MAX, 0,0);
	bls_l2cap_initMtuBuffer(mtu_s_rx_fifo, MTU_S_BUFF_SIZE_MAX, mtu_s_tx_fifo, MTU_S_BUFF_SIZE_MAX);
	my_gatt_init();  //gatt initialization
	#if (BLE_MASTER_SIMPLE_SDP_ENABLE)
		host_att_register_idle_func (main_idle_loop);
	#endif
	blc_gatt_register_data_handler(app_gatt_data_handler);


	//SMP initialization
	#if (BLE_SLAVE_SMP_ENABLE)  //Slave SMP Enable
		blc_smp_setSecurityLevel_slave(Unauthenticated_Paring_with_Encryption);  //LE_Security_Mode_1_Level_2
	#else
		blc_smp_setSecurityLevel_slave(No_Security);
	#endif

	#if (BLE_MASTER_SMP_ENABLE) //Master SMP Enable
		blc_smp_setSecurityLevel_master(Unauthenticated_Paring_with_Encryption);  //LE_Security_Mode_1_Level_2
	#else
		blc_smp_setSecurityLevel_master(No_Security);
		//user_master_host_pairing_management_init(); 		//TeLink referenced paring&bonding without standard paring in BLE Spec
	#endif

	blc_smp_smpParamInit();


	//host(GAP/SMP/GATT/ATT) event process: register host event callback and set event mask
	blc_gap_registerHostEventHandler( app_host_event_callback );
	blc_gap_setEventMask( GAP_EVT_MASK_SMP_PARING_BEAGIN 			|  \
						  GAP_EVT_MASK_SMP_PARING_SUCCESS   		|  \
						  GAP_EVT_MASK_SMP_PARING_FAIL				|  \
						  GAP_EVT_MASK_SMP_SECURITY_PROCESS_DONE);


	////// ADV and Scan CONFIG  //////////
	blc_llms_setAdvData( (u8 *)tbl_advData, sizeof(tbl_advData) );
	blc_llms_setScanRspData( (u8 *)tbl_scanRsp, sizeof(tbl_scanRsp));
	blc_llms_setAdvParam(  ADV_INTERVAL_30MS, ADV_INTERVAL_30MS,
						   ADV_TYPE_CONNECTABLE_UNDIRECTED, OWN_ADDRESS_PUBLIC,
						   0,  NULL,
						   BLT_ENABLE_ADV_ALL, ADV_FP_NONE);
	blc_llms_setAdvEnable(1);  //ADV enable


	blc_llms_setScanParameter(SCAN_TYPE_PASSIVE, SCAN_INTERVAL_200MS, SCAN_WINDOW_200MS, OWN_ADDRESS_PUBLIC, SCAN_FP_ALLOW_ADV_ANY);
	blc_llms_setScanEnable (BLC_SCAN_ENABLE, DUP_FILTER_DISABLE);
}




#if 0
if(A_connTick && A_tick && clock_time_exceed(A_tick, 5000*1000)){
	//blc_gatt_pushFindInformationRequest(A_connHandle, 0x0001, 0xFFFF);

	//blc_gatt_pushFindByTypeValueRequest(A_connHandle, 0x0001, 0xFFFF, 0x2800, (u8*)&gap_server_UUID, sizeof(gap_server_UUID));
    //blc_gatt_pushFindByTypeValueRequest(A_connHandle, 0x0001, 0xFFFF, 0x2803, (u8*)&my_devNameCharVal, sizeof(my_devNameCharVal));
    //blc_gatt_pushFindByTypeValueRequest(A_connHandle, 0x0001, 0xFFFF, 0x2A00, (u8*)&my_devName, sizeof(my_devName));

	//blc_gatt_pushReadByTypeRequest(A_connHandle, 0x0001, 0xFFFF, (u8*)&tlUUID, sizeof(tlUUID));

	//blc_gatt_pushReadRequest(A_connHandle, 3);

	//blc_gatt_pushReadBlobRequest(A_connHandle, 3, 1);

	//blc_gatt_pushReadByGroupTypeRequest(A_connHandle, 0x0001, 0xFFFF, (u8*)&serverDeclare, sizeof(serverDeclare));

	A_tick = clock_time()|1;
}
#endif

u32 start_tick = 1;
u16 startHandle = 1;

u16 gapServerUUID = 0x1800;
u8 my_devNameCharVal[5] = {
	CHAR_PROP_READ | CHAR_PROP_NOTIFY,
	U16_LO(GenericAccess_DeviceName_DP_H), U16_HI(GenericAccess_DeviceName_DP_H),
	U16_LO(GATT_UUID_DEVICE_NAME), U16_HI(GATT_UUID_DEVICE_NAME)
};
u8 otaUUID[] = TELINK_SPP_DATA_OTA;
u16 serverDeclarationUUID = 0x2800;
static const u16 my_characterUUID = GATT_UUID_CHARACTER;


//// device Information  attribute values
static const u8 my_PnCharVal[5] = {
	CHAR_PROP_READ,
	U16_LO(DeviceInformation_pnpID_DP_H), U16_HI(DeviceInformation_pnpID_DP_H),
	U16_LO(CHARACTERISTIC_UUID_PNP_ID), U16_HI(CHARACTERISTIC_UUID_PNP_ID)
};

static const u8 my_PnPtrs [] = {0x02, 0x8a, 0x24, 0x66, 0x82, 0x01, 0x00};

#define TEST_FIND_INFO_REQ            1
#define TEST_FIND_BY_TYPE_VALUE_REQ   2
#define TEST_READ_BY_TYPE_REQ         3
#define TEST_READ_REQ                 4
#define TEST_READ_BLOB_REQ            5
#define TEST_READ_BY_GROUP_TYPE_REQ   6

#define TEST_API  TEST_READ_BY_GROUP_TYPE_REQ



/** --------------------NOTE---------------------------------------------------
 * the feature test need hardware and firmware follow:
 * Hardware:
 *         1. EVK:    as 4 Master 3 slave device
 *         2. dongle: as slave device
 * Firmware:
 *         1. you need compile this project to gain bin file, then burn the bin file to EVK
 *         2. you need compile "ble sample project" to gain bin file, then burn the bin file to dongle.
 */
void feature_gatt_api_test_mainloop(void)
{
	switch(TEST_API)
	{
	case TEST_FIND_INFO_REQ:
		for(int i=0; i<MASTER_MAX_NUM; i++){
			if(connHandleTbl[i] && clock_time_exceed(start_tick, 10000*1000)){
				blc_gatt_pushFindInformationRequest(connHandleTbl[i], 0x0001, 0xFFFF);
				start_tick = clock_time();
			}
		}
		break;
	case TEST_FIND_BY_TYPE_VALUE_REQ:
		for(int i=0; i<MASTER_MAX_NUM; i++){
			if(connHandleTbl[i] && clock_time_exceed(start_tick, 10000*1000)){
#if(0)
				blc_gatt_pushFindByTypeValueRequest(connHandleTbl[i], 0x0001, 0xFFFF, GATT_UUID_PRIMARY_SERVICE, (u8*)&gapServerUUID, sizeof(gapServerUUID));
#elif(0)
				blc_gatt_pushFindByTypeValueRequest(connHandleTbl[i], 0x0001, 0xFFFF, GATT_UUID_CHARACTER, (u8*)&my_PnCharVal, sizeof(my_PnCharVal));
#elif(1)
				blc_gatt_pushFindByTypeValueRequest(connHandleTbl[i], 0x0001, 0xFFFF, CHARACTERISTIC_UUID_PNP_ID, (u8*)&my_PnPtrs, sizeof(my_PnPtrs));
#endif

				start_tick = clock_time();
			}
		}
		break;
	case TEST_READ_BY_TYPE_REQ:
		for(int i=0; i<MASTER_MAX_NUM; i++){
			if(connHandleTbl[i] && clock_time_exceed(start_tick, 10000*1000)){
				blc_gatt_pushReadByTypeRequest(connHandleTbl[i], 0x0001, 0xFFFF, (u8*)&otaUUID, sizeof(otaUUID));
				start_tick = clock_time();
			}
		}
		break;
	case TEST_READ_REQ:
		for(int i=0; i<MASTER_MAX_NUM; i++){
			if(connHandleTbl[i] && clock_time_exceed(start_tick, 10000*1000)){
				blc_gatt_pushReadRequest(connHandleTbl[i], 3);//device name
				start_tick = clock_time();
			}
		}
		break;
	case TEST_READ_BLOB_REQ:
		for(int i=0; i<MASTER_MAX_NUM; i++){
			if(connHandleTbl[i] && clock_time_exceed(start_tick, 10000*1000)){
				blc_gatt_pushReadBlobRequest(connHandleTbl[i], 3, 1);//device name
				start_tick = clock_time();
			}
		}
		break;
	case TEST_READ_BY_GROUP_TYPE_REQ:
		for(int i=0; i<MASTER_MAX_NUM; i++){
			if(connHandleTbl[i] && clock_time_exceed(start_tick, 10000*1000)){

#if (0)
				blc_gatt_pushReadByGroupTypeRequest(connHandleTbl[i], 0x0001, 0xFFFF,
						(u8*)&serverDeclarationUUID, sizeof(serverDeclarationUUID));
#elif(1)
				blc_gatt_pushReadByGroupTypeRequest(connHandleTbl[i], 0x0001, 0xFFFF,
						(u8*)&my_characterUUID, sizeof(my_characterUUID));
#endif
				start_tick = clock_time();
			}
		}
		break;
	default:break;
	}
}

#endif
