/********************************************************************************************************
 * @file     feature_security.c
 *
 * @brief    for TLSR chips
 *
 * @author	 BLE Group
 * @date     Nov. 18, 2019
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided. 
 *           
 *******************************************************************************************************/
#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"
#include "vendor/common/blt_common.h"

#include "app.h"

#include "app_att.h"
#include "app_device.h"

#include "application/keyboard/keyboard.h"
#include "application/usbstd/usbkeycode.h"


#if (FEATURE_TEST_MODE == TEST_SMP_SECURITY)



#ifndef	LL_MD_TEST_ENABLE
#define	LL_MD_TEST_ENABLE	0
#endif

#ifndef	LL_RX_OVERFLOW_TEST_ENABLE
#define	LL_RX_OVERFLOW_TEST_ENABLE	0
#endif


#if (LL_MD_TEST_ENABLE)
	#define		BLE_USR_RX_FIFO_NUM			32
	#define		BLE_M_USR_TX_FIFO_NUM		32
	#define		BLE_S_USR_TX_FIFO_NUM		32
#else
	#define		BLE_USR_RX_FIFO_NUM			16
	#define		BLE_M_USR_TX_FIFO_NUM		8
	#define		BLE_S_USR_TX_FIFO_NUM		8
#endif


#if (LL_RX_OVERFLOW_TEST_ENABLE)
static u32 block_tick = 0;
#endif


#if (UI_LED_ENABLE)
	#define	GPIO_LED_RED			GPIO_PD5
	#define	GPIO_LED_WHITE			GPIO_PD4
	#define	GPIO_LED_GREEN			GPIO_PD3
	#define	GPIO_LED_BLUE			GPIO_PD2

	#define LED_ON_LEVAL 			1 		//gpio output high voltage to turn on led



	const u32 gpio_table[4] = {GPIO_LED_RED, GPIO_LED_WHITE, GPIO_LED_GREEN, GPIO_LED_BLUE};

#endif



MYFIFO_INIT(blt_rxfifo, 64, BLE_USR_RX_FIFO_NUM);


/*Note: different connections have their own TX FIFO, master and slave set independently
 * Tx FIFO size = Extra_Len(10) + then align 4bytes
 */
MULTI_CONN_FIFO_INIT(blt_m_txfifo, 40, BLE_M_USR_TX_FIFO_NUM, MASTER_MAX_NUM);  //master connection TX FIFO,  40*8*MASTER_MAX_NUM
MULTI_CONN_FIFO_INIT(blt_s_txfifo, 40, BLE_S_USR_TX_FIFO_NUM, SLAVE_MAX_NUM);   //slave  connection TX FIFO,  40*8*SLAVE_MAX_NUM

/*Note:
 * MTU Buff size = Extra_Len(10)+ ATT_MTU_MAX(23) then align 4bytes
 */
#if (1) //if support LE Secure Connections, MTU more than 65
	#define	MTU_M_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(65)
	#define MTU_S_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(65)
#else
	#define	MTU_M_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(23)
	#define MTU_S_BUFF_SIZE_MAX			ATT_ALLIGN4_DMA_BUFF(23)
#endif

_attribute_data_retention_	u8 mtu_m_rx_fifo[MASTER_MAX_NUM * MTU_M_BUFF_SIZE_MAX];

_attribute_data_retention_	u8 mtu_s_rx_fifo[SLAVE_MAX_NUM * MTU_S_BUFF_SIZE_MAX];
_attribute_data_retention_	u8 mtu_s_tx_fifo[SLAVE_MAX_NUM * MTU_S_BUFF_SIZE_MAX];



int	master_smp_pending = 0; 		// SMP: security & encryption;

u16	app_conn_smp_flag[BLMS_MAX_CONN_NUM];


const u8	tbl_advData[] = {
	 0x08, 0x09, 'M', 'S', '-', 'R', 'o', 'l', 'e',
	 0x02, 0x01, 0x05, 							// BLE limited discoverable mode and BR/EDR not supported
	 0x03, 0x19, 0x80, 0x01, 					// 384, Generic Remote Control, Generic category
	 0x05, 0x02, 0x12, 0x18, 0x0F, 0x18,		// incomplete list of service class UUIDs (0x1812, 0x180F)
};

const u8	tbl_scanRsp [] = {
	 0x08, 0x09, 'M', 'S', '-', 'R', 'o', 'l', 'e',
};



// LE_Security_Mode_1_Level_1
#define		SECURITY_MODE1_LEVEL1_IO			0xff
#define		SECURITY_MODE1_LEVEL1_IO_S			0xff
#define		SECURITY_MODE1_LEVEL1_IO_M			0xff

// LE_Security_Mode_1_Level_2,3
#define		SECURITY_MODE1_LEVEL23_IO_DO		0
#define		SECURITY_MODE1_LEVEL23_IO_DO_S		0
#define		SECURITY_MODE1_LEVEL23_IO_DO_M		0
#define		SECURITY_MODE1_LEVEL23_IO_DYN		1
#define		SECURITY_MODE1_LEVEL23_IO_DYN_S		1
#define		SECURITY_MODE1_LEVEL23_IO_DYN_M 	1
#define		SECURITY_MODE1_LEVEL23_IO_KO		2
#define		SECURITY_MODE1_LEVEL23_IO_KO_S		2
#define		SECURITY_MODE1_LEVEL23_IO_KO_M		2
#define		SECURITY_MODE1_LEVEL23_IO_NINO		3
#define		SECURITY_MODE1_LEVEL23_IO_NINO_S	3
#define		SECURITY_MODE1_LEVEL23_IO_NINO_M	3
#define		SECURITY_MODE1_LEVEL23_IO_KD		4
#define		SECURITY_MODE1_LEVEL23_IO_KD_S		4
#define		SECURITY_MODE1_LEVEL23_IO_KD_M		4

// LE_Security_Mode_1_Level_4
#define		SECURITY_MODE1_LEVEL4_IO_DO			5
#define		SECURITY_MODE1_LEVEL4_IO_DO_S		5
#define		SECURITY_MODE1_LEVEL4_IO_DO_M		5
#define		SECURITY_MODE1_LEVEL4_IO_DYN		6
#define		SECURITY_MODE1_LEVEL4_IO_DYN_S		6
#define		SECURITY_MODE1_LEVEL4_IO_DYN_M 		6
#define		SECURITY_MODE1_LEVEL4_IO_KO			7
#define		SECURITY_MODE1_LEVEL4_IO_KO_S		7
#define		SECURITY_MODE1_LEVEL4_IO_KO_M		7
#define		SECURITY_MODE1_LEVEL4_IO_NINO		8
#define		SECURITY_MODE1_LEVEL4_IO_NINO_S		8
#define		SECURITY_MODE1_LEVEL4_IO_NINO_M		8
#define		SECURITY_MODE1_LEVEL4_IO_KD 		9
#define		SECURITY_MODE1_LEVEL4_IO_KD_S		9
#define		SECURITY_MODE1_LEVEL4_IO_KD_M		9

//SC OOB_Authentication, not support


#define     SMP_CFG_USE_COMMON_API_EN           1

#if (SMP_CFG_USE_COMMON_API_EN)
	#define     SMP_TEST_MODE_C					8
#else
	#define     SMP_TEST_MODE_S					SECURITY_MODE1_LEVEL23_IO_NINO_S
	#define     SMP_TEST_MODE_M					SECURITY_MODE1_LEVEL23_IO_NINO_M
#endif


#if (LL_MD_TEST_ENABLE)
	int muliti_ll_md_start = 0;
	u32 muliti_ll_md_stopTick = 0;
	u8  muliti_ll_md_stop = 0;
	u8  pushTxStopStart[BLMS_MAX_CONN_NUM];
	const u8 zero_all[BLMS_MAX_CONN_NUM] = { 0};
	#define TEST_DATA_LEN		20
	u8	app_test_data[BLMS_MAX_CONN_NUM][TEST_DATA_LEN];
	u8	app_test_start[BLMS_MAX_CONN_NUM];
	u32 device_connection_tick[BLMS_MAX_CONN_NUM];
	u8 write_cmd_seq_num_next;
	u8 notify_seq_num_next;
	u8 currRcvdSeqNo[BLMS_MAX_CONN_NUM] = {0};
	int AA_dbg_write_cmd_err = 0;
	int AA_dbg_notify_err = 0;
#endif


int	master_pairing_enable = 0;
int master_unpair_enable = 0;
int master_disconnect_connhandle;   //mark the master connection which is in un_pair disconnection flow


u8 	key_type;
int key_not_released;
u8  digital_key_cnt;
u8  tk_input[6];


static const u8 vk_dig_map[10] = {
	VK_0, VK_1, VK_2, VK_3, VK_4, VK_5, VK_6, VK_7, VK_8, VK_9,
};



int app_controller_event_callback (u32 h, u8 *p, int n);
int app_host_event_callback (u32 h, u8 *para, int n);
int app_gatt_data_handler (u16 connHandle, u8 *pkt);





int key_not_released;




void key_change_proc(void)
{
	u8 key0 = kb_event.keycode[0];
	u8 key_value;
	u8 key_buf[8] = {0,0,0,0,0,0,0,0};

	key_not_released = 1;
	if (kb_event.cnt >= 2){   //two or more key press, do  not process

	}
	else if(kb_event.cnt == 1){
		key_value = key0;
		if(key0 == BTN_PAIR)   //Manual pair triggered by Key Press
		{
			master_pairing_enable = 1;
		}
		if(key_value >= 0xf0 ){
			key_type = CONSUMER_KEY;
			u16 consumer_key;
			if(key0 == CR_VOL_UP){  	//volume up
				consumer_key = MKEY_VOL_UP;
				
			#if (LL_MD_TEST_ENABLE)
				printf("vol+\n");

				if(!muliti_ll_md_stopTick || (muliti_ll_md_stopTick && clock_time_exceed(muliti_ll_md_stopTick , 4000000))){
					muliti_ll_md_stopTick = 0;

					if(!muliti_ll_md_start){ //if already start LL MD test,do nothing
						for(int i=0; i< BLMS_MAX_CONN_NUM; i++)
						{
							if(app_test_start[i]){  //connection state
								muliti_ll_md_start = 1;

								///////////////set start flg///////////////
								memset(app_test_data, 0, sizeof(app_test_data));
								memset(currRcvdSeqNo, 0, sizeof(currRcvdSeqNo));
								for(int i=0; i< BLMS_MAX_CONN_NUM; i++){
									app_test_data[i][4] = 1; //start
								}
								break;
							}
						}
						printf("Start LL MD test %s\n", !muliti_ll_md_start? "failed":"succeed");
					}
				}
			#endif
			}
			else if(key0 == CR_VOL_DN){ //volume down
				consumer_key = MKEY_VOL_DN;
				
			#if (LL_MD_TEST_ENABLE)
				printf("vol-\n");

				if(muliti_ll_md_start){ //if already stop LL MD test,do nothing
					muliti_ll_md_start = 0;
					muliti_ll_md_stop = 1;
					memcpy(pushTxStopStart, app_test_start, sizeof(pushTxStopStart));
				}
			#endif
			}

			/*Here is just Telink Demonstration effect. Cause the demo board has limited key to use, when Vol+/Vol- key pressed, we
			send media key "Vol+" or "Vol-" to master for all slave in connection.
			For users, you should known that this is not a good method, you should manage your device and GATT data transfer
			according to  conn_dev_list[]
			 * */
			for(int i=0; i < conn_slave_num; i++){ //slave number in connection is "conn_slave_num"
				//slave index in device list table is from MASTER_MAX_NUM to MASTER_MAX_NUM + conn_slave_num - 1
				blc_gatt_pushHandleValueNotify ( conn_dev_list[MASTER_MAX_NUM + i].conn_handle, HID_CONSUME_REPORT_INPUT_DP_H, (u8 *)&consumer_key, 2);
			}
		}
		else
		{
			if(key0 == BTN_PAIR)   //Manual pair triggered by Key Press
			{
				key_type = PAIR_UNPAIR_KEY;

				master_pairing_enable = 1;
				printf("pair\n");

			}
			else if(key0 == BTN_UNPAIR) //Manual un_pair triggered by Key Press
			{
				key_type = PAIR_UNPAIR_KEY;
				printf("unpair\n");
				/*Here is just Telink Demonstration effect. Cause the demo board has limited key to use, only one "un_pair" key is
				 available. When "un_pair" key pressed, we will choose and un_pair one device in connection state */
				if(conn_master_num){ //at least 1 master connection exist

					if(!master_disconnect_connhandle){  //if one master un_pair disconnection flow not finish, here new un_pair not accepted
						// We choose the earliest connected device to un_pair, conn_dev_list[0] is the earliest connected device.
						// Attention: when conn_master_num none zero, conn_dev_list[0].conn_state is definite 1, no need to judge
						master_unpair_enable = conn_dev_list[0].conn_handle;  //mark connHandle on master_unpair_enable
					}
				}
			}
			else{
				key_type = KEYBOARD_KEY;
				key_buf[2] = key_value;

				//Here is just Telink Demonstration effect. for all slave in connection, send release for previous "Vol+" or "Vol-" to master
				for(int i=0; i < conn_slave_num; i++){ //slave number in connection is "conn_slave_num"
					//slave index in device list table is from MASTER_MAX_NUM to MASTER_MAX_NUM + conn_slave_num - 1
					blc_gatt_pushHandleValueNotify (  conn_dev_list[MASTER_MAX_NUM + i].conn_handle, HID_NORMAL_KB_REPORT_INPUT_DP_H, key_buf, 8); //release
				}

				#if ((SMP_CFG_USE_COMMON_API_EN && (SMP_TEST_MODE_C != SECURITY_MODE1_LEVEL1_IO)) || \
					((!SMP_CFG_USE_COMMON_API_EN) && (SMP_TEST_MODE_M != SECURITY_MODE1_LEVEL1_IO_M || SMP_TEST_MODE_S != SECURITY_MODE1_LEVEL1_IO_S)))
					u16 currSmpConnHandle = 0;
					for(int i=0; i< BLMS_MAX_CONN_NUM; i++)
					{
						if(app_conn_smp_flag[i]){
							currSmpConnHandle = app_conn_smp_flag[i];
							break;
						}
					}

					if(blc_smp_isWaitingToSetPasskeyEntry(currSmpConnHandle)){
						if( key_value <= VK_0 && key_value >= VK_1 && digital_key_cnt < 6){//key: 0~9
							int i;
							for(i = 0; i<10; i++){
								if(vk_dig_map[i] == key_value){
									printf("%d\n", i);
									tk_input[digital_key_cnt++] = i;
									break;
								}
							}
						}
						else if( key_value == VK_ENTER){// key: Enter/OK
							if(digital_key_cnt >= 6){
								digital_key_cnt = 0;
								u32 pincode = tk_input[0]*100000 + tk_input[1]*10000 + \
											  tk_input[2]*1000 + tk_input[3]*100 + tk_input[4]*10 + tk_input[5];
								printf("TK 6-dig value:%d\n", pincode);
								blc_smp_setTK_by_PasskeyEntry(currSmpConnHandle, pincode);
								app_conn_smp_flag[currSmpConnHandle & CONN_IDX_MASK] = 0;
							}
						}
					}
					else if(blc_smp_isWaitingToCfmNumericComparison(currSmpConnHandle)){
						if(key_value == VK_Y){
							blc_smp_setNumericComparisonResult(currSmpConnHandle, 1);// YES
							app_conn_smp_flag[currSmpConnHandle & CONN_IDX_MASK] = 0;
							printf("[0x%x]confirmed YES\n", currSmpConnHandle);
						}
						else if(key_value == VK_N){
							blc_smp_setNumericComparisonResult(currSmpConnHandle, 0);// NO
							app_conn_smp_flag[currSmpConnHandle & CONN_IDX_MASK] = 0;
							printf("[0x%x]confirmed NO\n", currSmpConnHandle);
						}
					}
				#endif
			}
		}
	}
	else   //kb_event.cnt == 0,  key release
	{
		key_not_released = 0;
		if(key_type == CONSUMER_KEY)
		{
			u16 consumer_key = 0;
			//Here is just Telink Demonstration effect. for all slave in connection, send release for previous "Vol+" or "Vol-" to master
			for(int i=0; i < conn_slave_num; i++){ //slave number in connection is "conn_slave_num"
				//slave index in device list table is from MASTER_MAX_NUM to MASTER_MAX_NUM + conn_slave_num - 1
				blc_gatt_pushHandleValueNotify (  conn_dev_list[MASTER_MAX_NUM + i].conn_handle, HID_CONSUME_REPORT_INPUT_DP_H, (u8 *)&consumer_key, 2);
			}
		}
		else if(key_type == KEYBOARD_KEY)
		{
			key_buf[2] = 0;
			//Here is just Telink Demonstration effect. for all slave in connection, send release for previous "Vol+" or "Vol-" to master
			for(int i=0; i < conn_slave_num; i++){ //slave number in connection is "conn_slave_num"
				//slave index in device list table is from MASTER_MAX_NUM to MASTER_MAX_NUM + conn_slave_num - 1
				blc_gatt_pushHandleValueNotify (  conn_dev_list[MASTER_MAX_NUM + i].conn_handle, HID_NORMAL_KB_REPORT_INPUT_DP_H, key_buf, 8); //release
			}
		}
		else if(key_type == PAIR_UNPAIR_KEY){
			if(master_pairing_enable){
				master_pairing_enable = 0;
			}

			if(master_unpair_enable){
				master_unpair_enable = 0;
			}
		}
	}
}



static u32 keyScanTick = 0;


void proc_keyboard (u8 e, u8 *p, int n)
{
	if(clock_time_exceed(keyScanTick, 8000)){
		keyScanTick = clock_time();
	}
	else{
		return;
	}

	kb_event.keycode[0] = 0;
	int det_key = kb_scan_key (0, 1);

	if (det_key){
		key_change_proc();
	}
}














int app_le_adv_report_event_handle(u8 *p)
{
	event_adv_report_t *pa = (event_adv_report_t *)p;
	s8 rssi = pa->data[pa->len];



	/*********************** Master Create connection demo  ********************/
	#if ((SMP_CFG_USE_COMMON_API_EN && (SMP_TEST_MODE_C != SECURITY_MODE1_LEVEL1_IO)) || \
		((!SMP_CFG_USE_COMMON_API_EN) && (SMP_TEST_MODE_M != SECURITY_MODE1_LEVEL1_IO_M)))
		if(master_smp_pending){ 	 //if previous connection SMP not finish, can not create a new connection
			return 1;
		}
	#endif

	if (master_disconnect_connhandle){ //one master connection is in un_pair disconnection flow, do not create a new one
		return 1;
	}



	int master_auto_connect = 0;
	int user_manual_paring = 0;

	#if ((SMP_CFG_USE_COMMON_API_EN && (SMP_TEST_MODE_C != SECURITY_MODE1_LEVEL1_IO)) || \
		((!SMP_CFG_USE_COMMON_API_EN) && (SMP_TEST_MODE_M != SECURITY_MODE1_LEVEL1_IO_M)))
		master_auto_connect = blc_smp_searchBondingSlaveDevice_by_PeerMacAddress(pa->adr_type, pa->mac);
	#endif

	//manual paring methods 1: key press triggers
	user_manual_paring = master_pairing_enable && (rssi > -56);  //button trigger pairing(RSSI threshold, short distance)


	if(master_auto_connect || user_manual_paring){

		blc_llms_createConnection( SCAN_INTERVAL_100MS, SCAN_INTERVAL_100MS, INITIATE_FP_ADV_SPECIFY,  \
								   pa->adr_type, pa->mac, BLE_ADDR_PUBLIC, \
								   CONN_INTERVAL_31P25MS, CONN_INTERVAL_31P25MS, 0, CONN_TIMEOUT_4S, \
								   0, 0xFFFF);
	}

	return 0;
}



int app_le_connection_complete_event_handle(u8 *p)
{
	event_connection_complete_t *pCon = (event_connection_complete_t *)p;

	if(pCon->status == BLE_SUCCESS){

		memset(&cur_conn_device, 0, sizeof(dev_char_info_t));

		//save current connect address type and address and conn_handle
		cur_conn_device.conn_handle = pCon->handle;
		cur_conn_device.peer_adrType = pCon->peer_adr_type;
		memcpy(cur_conn_device.peer_addr, pCon->mac, 6);

		dev_char_info_insert( &cur_conn_device );

		if( pCon->handle & BLM_CONN_HANDLE ){ // master connection handle, process SMP
			#if ((SMP_CFG_USE_COMMON_API_EN && (SMP_TEST_MODE_C != SECURITY_MODE1_LEVEL1_IO)) || \
				((!SMP_CFG_USE_COMMON_API_EN) && (SMP_TEST_MODE_M != SECURITY_MODE1_LEVEL1_IO_M)))
				master_smp_pending = pCon->handle; // this connection need SMP
			#endif
		}
		
	#if (LL_MD_TEST_ENABLE)
		u8 conn_idx = pCon->handle & CONN_IDX_MASK;
		device_connection_tick[conn_idx] = clock_time() | 1;
		app_test_data[conn_idx][0] = 0;
		currRcvdSeqNo[conn_idx] = 0;
	#endif

		printf("*[0x%x]Connected complete event*\n", pCon->handle);

	}

	return 0;
}





int 	app_disconnect_event_handle(u8 *p)
{
	event_disconnection_t	*pd = (event_disconnection_t *)p;

	//terminate reason
	if(pd->reason == HCI_ERR_CONN_TIMEOUT){  	//connection timeout

	}
	else if(pd->reason == HCI_ERR_REMOTE_USER_TERM_CONN){  	//peer device send terminate command on link layer

	}
	//master host disconnect( blm_ll_disconnect(current_connHandle, HCI_ERR_REMOTE_USER_TERM_CONN) )
	else if(pd->reason == HCI_ERR_CONN_TERM_BY_LOCAL_HOST){

	}
	else{

	}

	printf("*[0x%x]Disconnected event:0x%x *\n", pd->connHandle, pd->reason);

	//if previous connection SMP & SDP not finished, clear flag
	#if ((SMP_CFG_USE_COMMON_API_EN && (SMP_TEST_MODE_C != SECURITY_MODE1_LEVEL1_IO)) || \
		((!SMP_CFG_USE_COMMON_API_EN) && (SMP_TEST_MODE_M != SECURITY_MODE1_LEVEL1_IO_M)))
		if(master_smp_pending == pd->connHandle){
			master_smp_pending = 0;
		}

		if(master_disconnect_connhandle == pd->connHandle){  //un_pair disconnection flow finish, clear flag
			master_disconnect_connhandle = 0;
		}
	#endif

	dev_char_info_delete_by_connhandle(pd->connHandle);
	u8 conn_idx = pd->connHandle & CONN_IDX_MASK;
	app_conn_smp_flag[conn_idx] = 0;

#if (LL_MD_TEST_ENABLE)
	app_test_start[conn_idx]    = 0;
	device_connection_tick[conn_idx] = 0;
	memset(pushTxStopStart, 0, sizeof(pushTxStopStart));
#endif

	return 0;
}



//////////////////////////////////////////////////////////
// event call back
//////////////////////////////////////////////////////////
int app_controller_event_callback (u32 h, u8 *p, int n)
{
	if (h &HCI_FLAG_EVENT_BT_STD)		//Controller HCI event
	{
		u8 evtCode = h & 0xff;

		//------------ disconnect -------------------------------------
		if(evtCode == HCI_EVT_DISCONNECTION_COMPLETE)  //connection terminate
		{
			app_disconnect_event_handle(p);
		}
		else if(evtCode == HCI_EVT_LE_META)  //LE Event
		{
			u8 subEvt_code = p[0];

			//------hci le event: le connection complete event---------------------------------
			if (subEvt_code == HCI_SUB_EVT_LE_CONNECTION_COMPLETE)	// connection complete
			{
				app_le_connection_complete_event_handle(p);

			}
			//------hci le event: le connection establish event---------------------------------
			else if(subEvt_code == HCI_SUB_EVT_LE_CONNECTION_ESTABLISH)
			{
				//	blc_le_connection_establish_event_handle(p);
			}
			//--------hci le event: le adv report event ----------------------------------------
			else if (subEvt_code == HCI_SUB_EVT_LE_ADVERTISING_REPORT)	// ADV packet
			{
				app_le_adv_report_event_handle(p);
			}
			//------hci le event: le connection update complete event-------------------------------
			else if (subEvt_code == HCI_SUB_EVT_LE_CONNECTION_UPDATE_COMPLETE)	// connection update
			{

			}
		}
	}


	return 0;

}





int app_host_event_callback (u32 h, u8 *para, int n)
{
	u8 event = h & 0xFF;

	switch(event)
	{
		case GAP_EVT_SMP_PARING_BEAGIN:
		{
			gap_smp_paringBeginEvt_t *p = (gap_smp_paringBeginEvt_t *)para;
			/*
			 * 	JustWorks = 0, PK_Init_Dsply_Resp_Input = 1, PK_Resp_Dsply_Init_Input = 2, PK_BOTH_INPUT = 3,OOB_Authentication = 4, Numric_Comparison = 5,
			 */
			u8 TK_method[6][40] = {"JustWorks", "PK_Init_Dsply_Resp_Input", "PK_Resp_Dsply_Init_Input", "PK_BOTH_INPUT", "OOB_Authentication", "Numric_Comparison"};
			printf("[0x%x]Pairing begin (%s,TK-method:%s)\n", p->connHandle, p->secure_conn?"SC":"Legacy", TK_method[p->tk_method]);

			u8 conn_idx = p->connHandle & CONN_IDX_MASK;
			app_conn_smp_flag[conn_idx] = 0;
			digital_key_cnt = 0;
		}
		break;

		case GAP_EVT_SMP_PARING_SUCCESS:
		{
			gap_smp_paringSuccessEvt_t* p = (gap_smp_paringSuccessEvt_t*)para;
		    printf("[0x%x]Pairing success: bond%s (save %s)\n", p->connHandle, p->bonding ?"T":"F", p->bonding_result ? "OK" : "ERR");
		}
		break;

		case GAP_EVT_SMP_PARING_FAIL:
		{
			gap_smp_paringFailEvt_t* p = (gap_smp_paringFailEvt_t*)para;
			printf("[0x%x]Pairing failed:rsn:0x%x\n", p->connHandle, p->reason);
			u8 conn_idx = p->connHandle & CONN_IDX_MASK;
			app_conn_smp_flag[conn_idx] = 0;
		}
		break;

		case GAP_EVT_SMP_CONN_ENCRYPTION_DONE:
		{

		}
		break;


		case GAP_EVT_SMP_SECURITY_PROCESS_DONE:
		{
			gap_smp_connEncDoneEvt_t* p = (gap_smp_connEncDoneEvt_t*)para;
			printf("[0x%x]ENC done (%s)\n", p->connHandle, p->re_connect == SMP_STANDARD_PAIR ? "1stParing" :"reConnect");

			if( p->connHandle & BLM_CONN_HANDLE){  //master connection
				#if ((SMP_CFG_USE_COMMON_API_EN && (SMP_TEST_MODE_C != SECURITY_MODE1_LEVEL1_IO)) || \
					((!SMP_CFG_USE_COMMON_API_EN) && (SMP_TEST_MODE_M != SECURITY_MODE1_LEVEL1_IO_M)))
					if(master_smp_pending == p->connHandle){
						master_smp_pending = 0;
					}
				#endif
			}
		}
		break;



		case GAP_EVT_SMP_TK_DISPALY:
		{
			char pc[7];
			gap_smp_TkDisplayEvt_t* p = (gap_smp_TkDisplayEvt_t*)para;
			sprintf(pc, "%d", p->tk_pincode);
			printf("[0x%x]TK display:%s\n", p->connHandle, pc);
		}
		break;

		case GAP_EVT_SMP_TK_REQUEST_PASSKEY:
		{
			gap_smp_TkReqPassKeyEvt_t* p = (gap_smp_TkReqPassKeyEvt_t*)para;
			printf("[0x%x]TK Request passkey\n", p->connHandle);
			u8 conn_idx = p->connHandle & CONN_IDX_MASK;
			app_conn_smp_flag[conn_idx] |= p->connHandle;

		}
		break;

		case GAP_EVT_SMP_TK_REQUEST_OOB:
		{

		}
		break;

		case GAP_EVT_SMP_TK_NUMERIC_COMPARE:
		{
			gap_smp_TkDisplayEvt_t* p = (gap_smp_TkDisplayEvt_t*)para;
			printf("[0x%x]TK Numeric Comparison Vx:%d\n", p->connHandle, p->tk_pincode);

			u8 conn_idx = p->connHandle & CONN_IDX_MASK;
			app_conn_smp_flag[conn_idx] |= p->connHandle;
		}
		break;

		case GAP_EVT_ATT_EXCHANGE_MTU:
		{

		}
		break;

		case GAP_EVT_GATT_HANDLE_VLAUE_CONFIRM:
		{

		}
		break;

		default:
		break;
	}

	return 0;
}





int app_gatt_data_handler (u16 connHandle, u8 *pkt)
{

	if( connHandle & BLM_CONN_HANDLE)   //GATT data for Master
	{
			//so any ATT data before service discovery will be dropped
			dev_char_info_t* dev_info = dev_char_info_search_by_connhandle (connHandle);
			if(dev_info)
			{
				//-------	user process ------------------------------------------------
				rf_packet_att_t *pAtt = (rf_packet_att_t*)pkt;
				u16 attHandle = pAtt->handle0 | pAtt->handle1<<8;

				if(pAtt->opcode == ATT_OP_HANDLE_VALUE_NOTI)  //slave handle notify
				{
					if(attHandle == HID_CONSUME_REPORT_INPUT_DP_H){
						u16 consumer_key = pAtt->dat[0] | pAtt->dat[1]<<8;
						//Demo effect: when slave send Vol+/Vol- to master, LED GPIO toggle to show the result
					   #if (UI_LED_ENABLE)
						if(consumer_key == MKEY_VOL_UP){
							gpio_toggle(GPIO_LED_GREEN);
						}
						else if(consumer_key == MKEY_VOL_DN){
							gpio_toggle(GPIO_LED_BLUE);
						}
					   #endif
					}
					else if(attHandle == HID_NORMAL_KB_REPORT_INPUT_DP_H){   // Demo device(825x_ble_sample) Key Report AttHandle value is 29

					}
				#if (LL_MD_TEST_ENABLE)
					else if(attHandle == SPP_SERVER_TO_CLIENT_DP_H)
					{
						if(muliti_ll_md_start){
							u8 len = pAtt->l2capLen - 3;
							if(len > 0)
							{
								u8 seqNoRcv = pAtt->dat[0];
								u8 conn_idx = connHandle & CONN_IDX_MASK;
								if(seqNoRcv == currRcvdSeqNo[conn_idx]){
									currRcvdSeqNo[conn_idx]++;
								}
								else{
									AA_dbg_write_cmd_err = 1;
									printf("s2c:ERR[0x%x]:seq lost[seqNoRcv:%d][currRcvdSeqNo:%d]\n", connHandle, seqNoRcv, currRcvdSeqNo[conn_idx]);
									write_reg8(0x40000, 0x77);
									irq_disable();
									while(1);
								}
							}
						}
					}
				#endif
				}
				else if (pAtt->opcode == ATT_OP_HANDLE_VALUE_IND)
				{

				}
			}

	}
	else{   //GATT data for Slave


	}


	return 0;
}







///////////////////////////////////////////
void feature_security_test_init(void)
{
	//random number generator must be initiated here( in the beginning of user_init_nromal)
	//when deepSleep retention wakeUp, no need initialize again
	random_generator_init();  //this is must


///////////// BLE stack Initialization ////////////////
	//for 512K Flash, flash_sector_mac_address equals to 0x76000
	//for 1M   Flash, flash_sector_mac_address equals to 0xFF000
	u8  mac_public[6];
	u8  mac_random_static[6];
	blc_initMacAddress(flash_sector_mac_address, mac_public, mac_random_static);


	////// Controller Initialization  //////////
	blc_llms_initBasicMCU();
	blc_llms_initStandby_module(mac_public);						   //mandatory

	blc_llms_initAdvertising_module(); 	    //adv module: 		 mandatory for BLE slave,

	blc_llms_initScanning_module(); 		//scan module: 		 mandatory for BLE master,

	blc_llms_initMultiMasterMultiSlave_module();


	blc_llms_setMaxConnectionNumber( MASTER_MAX_NUM, SLAVE_MAX_NUM);



	rf_set_power_level_index (RF_POWER_P3p01dBm);


	////// HCI Initialization  //////////
	blc_hci_registerControllerDataHandler (blt_l2cap_pktHandler);

	blc_hci_registerControllerEventHandler(app_controller_event_callback); //controller hci event to host all processed in this func

	//bluetooth event
	blc_hci_setEventMask_cmd (HCI_EVT_MASK_DISCONNECTION_COMPLETE);

	//bluetooth low energy(LE) event
	blc_hci_le_setEventMask_cmd(		HCI_LE_EVT_MASK_CONNECTION_COMPLETE  \
									|	HCI_LE_EVT_MASK_ADVERTISING_REPORT \
									|   HCI_LE_EVT_MASK_CONNECTION_UPDATE_COMPLETE \
									|   HCI_LE_EVT_MASK_CONNECTION_ESTABLISH);  //connection establish: telink private event   TODO: delete this cause we will support standard HCI


	////// Host Initialization  //////////
	blc_gap_init();    //GAP initialization must be done before any other host feature initialization !!!
	//l2cap initialization
	blm_l2cap_initMtuBuffer(mtu_m_rx_fifo, MTU_M_BUFF_SIZE_MAX, 0,0);
	bls_l2cap_initMtuBuffer(mtu_s_rx_fifo, MTU_S_BUFF_SIZE_MAX, mtu_s_tx_fifo, MTU_S_BUFF_SIZE_MAX);
	my_gatt_init();  //gatt initialization
	blc_gatt_register_data_handler(app_gatt_data_handler);

	//host(GAP/SMP/GATT/ATT) event process: register host event callback and set event mask
	blc_gap_registerHostEventHandler( app_host_event_callback );
	blc_gap_setEventMask(0x3ffff); //all host event mask set

	//SMP initialization
#if (SMP_CFG_USE_COMMON_API_EN) //Use common API (master and slave share the same set of SMP parameters)
	//Slave SMP Enable
	#if( SMP_TEST_MODE_C == SECURITY_MODE1_LEVEL1_IO)
		blc_smp_setSecurityLevel(No_Security);
	#elif ( SMP_TEST_MODE_C == SECURITY_MODE1_LEVEL23_IO_NINO )
		//set security level: "LE_Security_Mode_1_Level_2"
		//blc_smp_setSecurityLevel(Unauthenticated_Paring_with_Encryption);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		//blc_smp_setBondingMode(Bondable_Mode);	// if not set, default is : Bondable_Mode
		//blc_smp_setIoCapability(IO_CAPABLITY_NO_IN_NO_OUT);	// if not set, default is : IO_CAPABILITY_NO_INPUT_NO_OUTPUT

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#elif ( SMP_TEST_MODE_C == SECURITY_MODE1_LEVEL4_IO_NINO )
		//set security level: "LE_Security_Mode_1_Level_2"
		//blc_smp_setSecurityLevel(Unauthenticated_Paring_with_Encryption);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		//blc_smp_setBondingMode(Bondable_Mode);	// if not set, default is : Bondable_Mode
		//blc_smp_setIoCapability(IO_CAPABLITY_NO_IN_NO_OUT);	// if not set, default is : IO_CAPABILITY_NO_INPUT_NO_OUTPUT
		blc_smp_setParingMethods(LE_Secure_Connection);
		blc_smp_setEcdhDebugMode(debug_mode);

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#elif ( SMP_TEST_MODE_C <= SECURITY_MODE1_LEVEL23_IO_KD )
		//set security level: "LE_Security_Mode_1_Level_3"
		blc_smp_setSecurityLevel(Authenticated_Paring_with_Encryption);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		blc_smp_enableAuthMITM(1);
		//blc_smp_setBondingMode(Bondable_Mode);	// if not set, default is : Bondable_Mode
		blc_smp_setIoCapability(SMP_TEST_MODE_C);	// if not set, default is : IO_CAPABILITY_NO_INPUT_NO_OUTPUT

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#else
		//set security level: "LE_Security_Mode_1_Level_4"
		//blc_smp_setSecurityLevel(Authenticated_LE_Secure_Connection_Paring_with_Encryption); //SC only
		blc_smp_setSecurityLevel(LE_Security_Mode_1);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		blc_smp_setSecurityParamters(Bondable_Mode, 1, LE_Secure_Connection, 0, 0, SMP_TEST_MODE_C - SECURITY_MODE1_LEVEL4_IO_DO);
		blc_smp_setEcdhDebugMode(debug_mode); //use debug mode for sniffer decryption

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#endif
#else //Use independent API (master and slave configure their own SMP parameters)
	//Slave SMP Enable
	#if( SMP_TEST_MODE_S == SECURITY_MODE1_LEVEL1_IO_S)
		blc_smp_setSecurityLevel_slave(No_Security);
	#elif ( SMP_TEST_MODE_S == SECURITY_MODE1_LEVEL23_IO_NINO_S )
		//set security level: "LE_Security_Mode_1_Level_2"
		//blc_smp_setSecurityLevel_slave(Unauthenticated_Paring_with_Encryption);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		//blc_smp_setBondingMode_slave(Bondable_Mode);	// if not set, default is : Bondable_Mode
		//blc_smp_setIoCapability_slave(IO_CAPABLITY_NO_IN_NO_OUT);	// if not set, default is : IO_CAPABILITY_NO_INPUT_NO_OUTPUT

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#elif ( SMP_TEST_MODE_S == SECURITY_MODE1_LEVEL4_IO_NINO_S )
		//set security level: "LE_Security_Mode_1_Level_2"
		//blc_smp_setSecurityLevel_slave(Unauthenticated_Paring_with_Encryption);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		//blc_smp_setBondingMode_slave(Bondable_Mode);	// if not set, default is : Bondable_Mode
		//blc_smp_setIoCapability_slave(IO_CAPABLITY_NO_IN_NO_OUT);	// if not set, default is : IO_CAPABILITY_NO_INPUT_NO_OUTPUT
		blc_smp_setParingMethods_slave(LE_Secure_Connection);
		blc_smp_setEcdhDebugMode(debug_mode);

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#elif ( SMP_TEST_MODE_S <= SECURITY_MODE1_LEVEL23_IO_KD_S )
		//set security level: "LE_Security_Mode_1_Level_3"
		blc_smp_setSecurityLevel_slave(Authenticated_Paring_with_Encryption);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		blc_smp_enableAuthMITM_slave(1);
		//blc_smp_setBondingMode_slave(Bondable_Mode);	// if not set, default is : Bondable_Mode
		blc_smp_setIoCapability_slave(SMP_TEST_MODE_S);	// if not set, default is : IO_CAPABILITY_NO_INPUT_NO_OUTPUT

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#else
		//set security level: "LE_Security_Mode_1_Level_4"
		//blc_smp_setSecurityLevel(Authenticated_LE_Secure_Connection_Paring_with_Encryption); //SC only
		blc_smp_setSecurityLevel_slave(LE_Security_Mode_1);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		blc_smp_setSecurityParamters_slave(Bondable_Mode, 1, LE_Secure_Connection, 0, 0, SMP_TEST_MODE_S - SECURITY_MODE1_LEVEL4_IO_DO_S);
		blc_smp_setEcdhDebugMode_slave(debug_mode); //use debug mode for sniffer decryption

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#endif

	//Master SMP Enable
	#if( SMP_TEST_MODE_M == SECURITY_MODE1_LEVEL1_IO_M)
		blc_smp_setSecurityLevel_master(No_Security);
	#elif ( SMP_TEST_MODE_M == SECURITY_MODE1_LEVEL23_IO_NINO_M )
		//set security level: "LE_Security_Mode_1_Level_2"
		//blc_smp_setSecurityLevel_master(Unauthenticated_Paring_with_Encryption);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		//blc_smp_setBondingMode_master(Bondable_Mode);	// if not set, default is : Bondable_Mode
		//blc_smp_setIoCapability_master(IO_CAPABLITY_NO_IN_NO_OUT);	// if not set, default is : IO_CAPABILITY_NO_INPUT_NO_OUTPUT

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#elif ( SMP_TEST_MODE_M == SECURITY_MODE1_LEVEL4_IO_NINO_M )
		//set security level: "LE_Security_Mode_1_Level_2"
		//blc_smp_setSecurityLevel_master(Unauthenticated_Paring_with_Encryption);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		//blc_smp_setBondingMode_master(Bondable_Mode);	// if not set, default is : Bondable_Mode
		//blc_smp_setIoCapability_master(IO_CAPABLITY_NO_IN_NO_OUT);	// if not set, default is : IO_CAPABILITY_NO_INPUT_NO_OUTPUT
		blc_smp_setParingMethods_master(LE_Secure_Connection);
		blc_smp_setEcdhDebugMode_master(debug_mode);

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#elif ( SMP_TEST_MODE_M <= SECURITY_MODE1_LEVEL23_IO_KD_M )
		//set security level: "LE_Security_Mode_1_Level_3"
		blc_smp_setSecurityLevel_master(Authenticated_Paring_with_Encryption);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		blc_smp_enableAuthMITM_master(1);
		//blc_smp_setBondingMode_master(Bondable_Mode);	// if not set, default is : Bondable_Mode
		blc_smp_setIoCapability_master(SMP_TEST_MODE_M);	// if not set, default is : IO_CAPABILITY_NO_INPUT_NO_OUTPUT

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#else
		//set security level: "LE_Security_Mode_1_Level_4"
		//blc_smp_setSecurityLevel(Authenticated_LE_Secure_Connection_Paring_with_Encryption); //SC only
		blc_smp_setSecurityLevel_master(LE_Security_Mode_1);  //if not set, default is : LE_Security_Mode_1_Level_2(Unauthenticated_Paring_with_Encryption)
		blc_smp_setSecurityParamters_master(Bondable_Mode, 1, LE_Secure_Connection, 0, 0, SMP_TEST_MODE_M - SECURITY_MODE1_LEVEL4_IO_DO_M);
		blc_smp_setEcdhDebugMode_master(debug_mode); //use debug mode for sniffer decryption

		blc_smp_configPairingRequestSending(PairReq_AUTO_SEND, PairReq_AUTO_SEND);
		// New paring: send security_request immediately after connection complete
		// reConnect:  send security_request 1000mS after connection complete. If master start paring or encryption before 1000mS timeout, slave do not send security_request.
		//if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
		blc_smp_configSecurityRequestSending(SecReq_IMM_SEND, SecReq_PEND_SEND, 1000); //if not set, default is:  send "security request" immediately after link layer connection established(regardless of new connection or reconnection )
	#endif
#endif


	blc_smp_smpParamInit();




	////// ADV and Scan CONFIG  //////////
	blc_llms_setAdvData( (u8 *)tbl_advData, sizeof(tbl_advData) );
	blc_llms_setScanRspData( (u8 *)tbl_scanRsp, sizeof(tbl_scanRsp));



	blc_llms_setAdvParam(  ADV_INTERVAL_30MS, ADV_INTERVAL_30MS,
											 ADV_TYPE_CONNECTABLE_UNDIRECTED, OWN_ADDRESS_PUBLIC,
											 0,  NULL,
											 BLT_ENABLE_ADV_ALL,
											 ADV_FP_NONE);
	blc_llms_setAdvEnable(1);  //ADV enable


	blc_llms_setScanParameter(SCAN_TYPE_PASSIVE, SCAN_INTERVAL_200MS, SCAN_WINDOW_200MS, OWN_ADDRESS_PUBLIC, SCAN_FP_ALLOW_ADV_ANY);
	blc_llms_setScanEnable (BLC_SCAN_ENABLE, DUP_FILTER_DISABLE);



//	blc_llms_setAdvCustomedChannel( 37, 37, 37);  //debug
#if (LL_MD_TEST_ENABLE) //test 1 master 1 slave 1st
	blc_llms_setMaxConnectionNumber(4, 3);
#else
	//blc_llms_setMaxConnectionNumber(4, 3);
#endif



#if (UI_LED_ENABLE)
	gpio_set_func(GPIO_LED_RED,   AS_GPIO);
	gpio_set_func(GPIO_LED_WHITE, AS_GPIO);
	gpio_set_func(GPIO_LED_GREEN,   AS_GPIO);
	gpio_set_func(GPIO_LED_BLUE, AS_GPIO);

	gpio_set_output_en(GPIO_LED_RED, 1);
	gpio_set_output_en(GPIO_LED_WHITE, 1);
	gpio_set_output_en(GPIO_LED_GREEN, 1);
	gpio_set_output_en(GPIO_LED_BLUE, 1);
#endif
}


void proc_master_role_unpair(void)
{
#if ((SMP_CFG_USE_COMMON_API_EN && (SMP_TEST_MODE_C != SECURITY_MODE1_LEVEL1_IO)) || \
	((!SMP_CFG_USE_COMMON_API_EN) && (SMP_TEST_MODE_M != SECURITY_MODE1_LEVEL1_IO_M)))
	//terminate and un_pair process, Telink demonstration effect: triggered by "un_pair" key press
	if(master_unpair_enable){
		dev_char_info_t* dev_char_info = dev_char_info_search_by_connhandle(master_unpair_enable); //connHandle has marked on on master_unpair_enable

		if( dev_char_info ){ //un_pair device in still in connection state

			if(blc_llms_disconnect(master_unpair_enable, HCI_ERR_REMOTE_USER_TERM_CONN) == BLE_SUCCESS){

				master_disconnect_connhandle = master_unpair_enable; //mark conn_handle

				master_unpair_enable = 0;  //every "un_pair" key can only triggers one connection disconnect

				// delete this device information(mac_address and distributed keys...) on FLash
				blc_smp_deleteBondingSlaveInfo_by_PeerMacAddress(dev_char_info->peer_adrType, dev_char_info->peer_addr);
			}
		}
		else{ //un_pair device can not find in device list, it's not connected now
			master_unpair_enable = 0;  //every "un_pair" key can only triggers one connection disconnect
		}
	}
#endif
}


int my_client_2_server_write_callback(u16 connHandle, void * p)
{
#if (LL_MD_TEST_ENABLE)
	rf_packet_att_data_t *req = (rf_packet_att_data_t*)p;

	if(muliti_ll_md_start){
		u8 seqNoRcv = req->dat[0];
		u8 conn_idx = connHandle & CONN_IDX_MASK;
		 if(currRcvdSeqNo[conn_idx] == seqNoRcv){
			 currRcvdSeqNo[conn_idx]++;
			 DBG_CHN11_TOGGLE;
		}
		else{
			AA_dbg_notify_err = 1;
			printf("c2s:ERR[0x%x]:seq lost[seqNoRcv:%d][currRcvdSeqNo:%d]\n", conn_idx, seqNoRcv, currRcvdSeqNo[conn_idx]);
			write_reg8(0x40000, 0x77);
			irq_disable();
			while(1);
		}
	}
#endif

	 return 1;
}


/////////////////////////////////////////////////////////////////////
// main loop flow
/////////////////////////////////////////////////////////////////////
void feature_security_test_mainloop (void)
{

	proc_keyboard( 0, 0, 0);
	proc_master_role_unpair();

#if (LL_MD_TEST_ENABLE)
	//for M4S3, connection handles are:
	//master: 0x0080/0x0081/0x0082/0x0083
	//slave:  0x0044/0x0045/0x0046
	for(int i=0; i< BLMS_MAX_CONN_NUM; i++)
	{
		if(device_connection_tick[i] && clock_time_exceed(device_connection_tick[i] , 1000000)){
			device_connection_tick[i]  = 0;

			app_test_start[i] = 1;
		}

		if(app_test_start[i]){  //connection state
			u16 connHandle;
			if(i < BLMS_MAX_CONN_MASTER_NUM){  //Master
				connHandle = BLM_CONN_HANDLE | i;

				if(muliti_ll_md_start){
					if(BLE_SUCCESS == blc_gatt_pushWriteCommand(connHandle,  SPP_CLIENT_TO_SERVER_DP_H, app_test_data[i], 20)){
						app_test_data[i][0] ++;
						DBG_CHN12_TOGGLE;
					}
				}
			}
			else{ //Slave
				connHandle = BLS_CONN_HANDLE | i;

				if(muliti_ll_md_start){
					if(BLE_SUCCESS == blc_gatt_pushHandleValueNotify(connHandle, SPP_SERVER_TO_CLIENT_DP_H, app_test_data[i], 20)){
						app_test_data[i][0] ++;
						DBG_CHN12_TOGGLE;
					}
				}
			}
		}
	}

	if(muliti_ll_md_stop){ //if already stop LL MD test,do nothing

		//for M4S3, connection handles are:
		//master: 0x0080/0x0081/0x0082/0x0083
		//slave:  0x0044/0x0045/0x0046
		for(int i=0; i< BLMS_MAX_CONN_NUM; i++)
		{
			if(pushTxStopStart[i]){
				app_test_data[i][4] = 0; //stop
				u16 connHandle;
				if(i < BLMS_MAX_CONN_MASTER_NUM){  //Master
					connHandle = BLM_CONN_HANDLE | i;
					if(BLE_SUCCESS == blc_gatt_pushWriteCommand(connHandle,  SPP_CLIENT_TO_SERVER_DP_H, app_test_data[i], 20)){
						pushTxStopStart[i] = 0;
					}
				}
				else{ //Slave
					connHandle = BLS_CONN_HANDLE | i;
					if(BLE_SUCCESS == blc_gatt_pushHandleValueNotify(connHandle, SPP_SERVER_TO_CLIENT_DP_H, app_test_data[i], 20)){
						pushTxStopStart[i] = 0;
					}
				}
			}
		}

		if(!memcmp(pushTxStopStart, zero_all, sizeof(pushTxStopStart))){
			muliti_ll_md_stopTick = clock_time()|1;
			muliti_ll_md_stop = 0;
			printf("Stop LL MD test %s\n", muliti_ll_md_start? "failed":"succeed");
		}
	}
#endif

#if (LL_RX_OVERFLOW_TEST_ENABLE)
	if(clock_time_exceed(block_tick, 4000000)){
		block_tick = clock_time();
	}
	else{
		return;
	}
	DBG_CHN14_TOGGLE;
	WaitMs(2000); //UI enter 1s block
#endif

}







#endif
